@@ -13,8 +13,14 @@ import os.path
 import re
 import sys
 
+def bytestostr(x):
+    return str(x)
 if sys.version > '3':
     unicode = str           #@ReservedAssignment
+    def bytestostr(x):
+        if type(x)==bytes:
+            return x.decode("latin1")
+        return str(x)
 
 
 def update_properties(props, filename):
@@ -43,7 +49,7 @@ def save_properties(props, filename):
             f.write(u(v))
         for name in sorted(props.keys()):
             value = props[name]
-            s = str(value).replace("'", "\\'")
+            s = bytestostr(value).replace("'", "\\'")
             w(name)
             w("=")
             quote_it = type(value) not in (bool, tuple, int)
@@ -55,7 +61,7 @@ def save_properties(props, filename):
             w("\n")
     print("updated %s with:" % filename)
     for k in sorted(props.keys()):
-        print("* %s = %s" % (str(k).ljust(20), u(props[k]).decode()))
+        print("* %s = %s" % (str(k).ljust(20), bytestostr(props[k])))
 
 def get_properties(filename):
     props = dict()
@@ -232,7 +238,7 @@ def record_build_info(is_build=True):
         for pkg in ("libc",
                     "vpx", "libvpx", "x264", "x265", "webp",
                     "avcodec", "avutil", "swscale",
-                    "nvenc4", "nvenc5",
+                    "nvenc4", "nvenc5", "nvenc6",
                     "x11", "xrandr", "xtst", "xfixes", "xkbfile", "xcomposite", "xdamage", "xext",
                     "gtk+-3.0", "pycairo", "pygobject-2.0", "pygtk-2.0", ):
             #fugly magic for turning the package atom into a legal variable name:
