@@ -10,7 +10,8 @@ def main():
     from xpra.codecs.nvenc4 import encoder
     test_nvenc.set_encoder_module(encoder)
     #test_nvenc.test_encode_one()
-    test_nvenc.test_context_leak()
+    #test_nvenc.test_context_leak()
+    test_nvenc.test_reconfigure()
     #test_nvenc.test_memleak()
     #test_nvenc.test_dimensions()
     #test_nvenc.test_perf()
