@@ -2,10 +2,9 @@
 function set_title(index, title) {
     var c = $('canvas.flot-base')[index];
     var canvas = c.getContext("2d");
-    var cx = c.width / 2;
-    canvas.font = "15px sans-serif";
+    canvas.font = "12px sans-serif";
     canvas.textAlign = 'center';
-    canvas.fillText(title, cx, 35);
+    canvas.fillText(title, 120, 25);
 }
 
 $(function() {
