@@ -89,9 +89,18 @@ class GLClientWindow(GTK2WindowBase):
                 self.remove(glarea)
             b.close()
             self._backing = None
-        self.iconify()
+        GTK2WindowBase.freeze(self)
 
 
+    def toggle_debug(self, *args):
+        b = self._backing
+        if not b:
+            return
+        if b.paint_box_line_width>0:
+            b.paint_box_line_width = 0
+        else:
+            b.paint_box_line_width = b.default_paint_box_line_width
+
     def magic_key(self, *args):
         b = self._backing
         if self.border:
