@@ -1,11 +1,10 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2015 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 from gtk import gdk
-import gobject
 import os
 
 from xpra.log import Logger
@@ -25,21 +24,21 @@ Superclass for PixmapBacking and GLBacking
 """
 class GTK2WindowBacking(GTKWindowBacking):
 
-    def init(self, w, h):
+    def init(self, *args):
         raise Exception("override me!")
 
 
     def paint_image(self, coding, img_data, x, y, width, height, options, callbacks):
         """ can be called from any thread """
-        if USE_PIL and has_codec("PIL"):
+        if USE_PIL and has_codec("dec_pillow"):
             return GTKWindowBacking.paint_image(self, coding, img_data, x, y, width, height, options, callbacks)
         #gdk needs UI thread:
-        gobject.idle_add(self.paint_pixbuf_gdk, coding, img_data, x, y, width, height, options, callbacks)
+        self.idle_add(self.paint_pixbuf_gdk, coding, img_data, x, y, width, height, options, callbacks)
         return  False
 
     def do_draw_region(self, x, y, width, height, coding, img_data, rowstride, options, callbacks):
         """ called as last resort when PIL is not available"""
-        gobject.idle_add(self.paint_pixbuf_gdk, coding, img_data, x, y, width, height, options, callbacks)
+        self.idle_add(self.paint_pixbuf_gdk, coding, img_data, x, y, width, height, options, callbacks)
 
 
     def paint_pixbuf_gdk(self, coding, img_data, x, y, width, height, options, callbacks):
@@ -54,7 +53,7 @@ class GTK2WindowBacking(GTKWindowBacking):
         pixbuf = loader.get_pixbuf()
         if not pixbuf:
             log.error("failed %s pixbuf=%s data len=%s" % (coding, pixbuf, len(img_data)))
-            fire_paint_callbacks(callbacks, False)
+            fire_paint_callbacks(callbacks, False, "failed to load a pixbuf from %i bytes of %s data" % (len(img_data), coding))
             return  False
         raw_data = pixbuf.get_pixels()
         rowstride = pixbuf.get_rowstride()
