@@ -9,6 +9,6 @@ log = Logger("gtk", "client")
 import os
 if os.name=="posix":
     try:
-        from xpra.x11.gtk3_x11 import gdk_display_source
-    except Exception:
-        log.warn("cannot import gtk3_x11 display source", exc_info=True)
+        from xpra.x11.gtk3 import gdk_display_source
+    except ImportError:
+        log.warn("cannot import gtk3 x11 display source", exc_info=True)
