@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -13,21 +13,14 @@ from gi.repository import GdkPixbuf             #@UnresolvedImport @UnusedImport
 
 from xpra.client.gtk3.cairo_backing import CairoBacking
 from xpra.client.gtk_base.gtk_client_window_base import GTKClientWindowBase, HAS_X11_BINDINGS
+from xpra.gtk_common.gtk_util import WINDOW_NAME_TO_HINT, WINDOW_EVENT_MASK, BUTTON_MASK
+from xpra.os_util import bytestostr
 from xpra.log import Logger
 log = Logger("gtk", "window")
 paintlog = Logger("paint")
+metalog = Logger("metadata")
 
 
-GTK3_WINDOW_EVENT_MASK = Gdk.EventMask.STRUCTURE_MASK | Gdk.EventMask.KEY_PRESS_MASK | Gdk.EventMask.KEY_RELEASE_MASK \
-            | Gdk.EventMask.POINTER_MOTION_MASK | Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK \
-            | Gdk.EventMask.PROPERTY_CHANGE_MASK | Gdk.EventMask.SCROLL_MASK
-
-GTK3_BUTTON_MASK = {Gdk.ModifierType.BUTTON1_MASK : 1,
-                    Gdk.ModifierType.BUTTON2_MASK : 2,
-                    Gdk.ModifierType.BUTTON3_MASK : 3,
-                    Gdk.ModifierType.BUTTON4_MASK : 4,
-                    Gdk.ModifierType.BUTTON5_MASK : 5}
-
 GTK3_SCROLL_MAP = {
                    Gdk.ScrollDirection.UP   : 4,
                    Gdk.ScrollDirection.DOWN : 5,
@@ -52,34 +45,16 @@ GTK3_OR_TYPE_HINTS = (Gdk.WindowTypeHint.DIALOG,
                       Gdk.WindowTypeHint.DND)
 
 
-GTK3_NAME_TO_HINT = {
-                "NORMAL"        : Gdk.WindowTypeHint.NORMAL,
-                "DIALOG"        : Gdk.WindowTypeHint.DIALOG,
-                "MENU"          : Gdk.WindowTypeHint.MENU,
-                "TOOLBAR"       : Gdk.WindowTypeHint.TOOLBAR,
-                "SPLASH"        : Gdk.WindowTypeHint.SPLASHSCREEN,
-                "UTILITY"       : Gdk.WindowTypeHint.UTILITY,
-                "DOCK"          : Gdk.WindowTypeHint.DOCK,
-                "DESKTOP"       : Gdk.WindowTypeHint.DESKTOP,
-                "DROPDOWN_MENU" : Gdk.WindowTypeHint.DROPDOWN_MENU,
-                "POPUP_MENU"    : Gdk.WindowTypeHint.POPUP_MENU,
-                "TOOLTIP"       : Gdk.WindowTypeHint.TOOLTIP,
-                "NOTIFICATION"  : Gdk.WindowTypeHint.NOTIFICATION,
-                "COMBO"         : Gdk.WindowTypeHint.COMBO,
-                "DND"           : Gdk.WindowTypeHint.DND
-                }
-
-
 """
 GTK3 version of the ClientWindow class
 """
-class ClientWindow(GTKClientWindowBase):
+class ClientWindow(GTKClientWindowBase, Gtk.Window):
 
-    WINDOW_EVENT_MASK   = GTK3_WINDOW_EVENT_MASK
-    BUTTON_MASK         = GTK3_BUTTON_MASK
+    WINDOW_EVENT_MASK   = WINDOW_EVENT_MASK
+    BUTTON_MASK         = BUTTON_MASK
     SCROLL_MAP          = GTK3_SCROLL_MAP
     OR_TYPE_HINTS       = GTK3_OR_TYPE_HINTS
-    NAME_TO_HINT        = GTK3_NAME_TO_HINT
+    NAME_TO_HINT        = WINDOW_NAME_TO_HINT
 
     WINDOW_STATE_FULLSCREEN = Gdk.WindowState.FULLSCREEN
     WINDOW_STATE_MAXIMIZED  = Gdk.WindowState.MAXIMIZED
@@ -89,22 +64,14 @@ class ClientWindow(GTKClientWindowBase):
     WINDOW_STATE_STICKY     = Gdk.WindowState.STICKY
 
 
-    def init_window(self, metadata):
-        #TODO: no idea how to do the window-type with gtk3
-        #maybe not even be possible..
-        if self._is_popup(metadata):
-            window_type = Gtk.WindowType.POPUP
-        else:
-            window_type = Gtk.WindowType.TOPLEVEL
-        Gtk.Window.__init__(self,
-                            type = window_type,
-                            decorated = self._is_decorated(metadata),
-                            app_paintable = True)
-        GTKClientWindowBase.init_window(self, metadata)
+    def do_init_window(self, window_type):
+        Gtk.Window.__init__(self, type = window_type)
         # tell KDE/oxygen not to intercept clicks
         # see: https://bugs.kde.org/show_bug.cgi?id=274485
         # does not work with gtk3? what the??
-        #self.set_data(strtobytes("_kde_no_window_grab"), 1)
+        # they moved it gobject, then removed it, unbelievable:
+        # https://bugzilla.gnome.org/show_bug.cgi?id=641944
+        #self.set_data("_kde_no_window_grab", 1)
         def motion(w, event):
             self.do_motion_notify_event(event)
         self.connect("motion-notify-event", motion)
@@ -133,18 +100,19 @@ class ClientWindow(GTKClientWindowBase):
 
 
     def xget_u32_property(self, target, name):
+        if HAS_X11_BINDINGS:
+            return GTKClientWindowBase.xget_u32_property(self, target, name)
+        #pure Gdk lookup:
         try:
-            if not HAS_X11_BINDINGS:
-                name_atom = Gdk.Atom.intern(name, False)
-                type_atom = Gdk.Atom.intern("CARDINAL", False)
-                prop = Gdk.property_get(target, name_atom, type_atom, 0, 9999, False)
-                if not prop or len(prop)!=3 or len(prop[2])!=1:
-                    return  None
-                log("xget_u32_property(%s, %s)=%s", target, name, prop[2][0])
-                return prop[2][0]
+            name_atom = Gdk.Atom.intern(name, False)
+            type_atom = Gdk.Atom.intern("CARDINAL", False)
+            prop = Gdk.property_get(target, name_atom, type_atom, 0, 9999, False)
+            if not prop or len(prop)!=3 or len(prop[2])!=1:
+                return  None
+            metalog("xget_u32_property(%s, %s)=%s", target, name, prop[2][0])
+            return prop[2][0]
         except Exception as e:
-            log.error("xget_u32_property error on %s / %s: %s", target, name, e)
-        return GTKClientWindowBase.xget_u32_property(self, target, name)
+            metalog.error("xget_u32_property error on %s / %s: %s", target, name, e)
 
     def is_mapped(self):
         return self.get_mapped()
@@ -185,7 +153,14 @@ class ClientWindow(GTKClientWindowBase):
         geom = Gdk.Geometry()
         mask = 0
         for k,v in hints.items():
+            k = bytestostr(k)
             if k in INT_FIELDS:
+                if k.find("width")>=0:
+                    v = self._client.sx(v)
+                elif k.find("height")>=0:
+                    v = self._client.sy(v)
+                elif k.find("size")>=0:
+                    v = self._client.sp(v)
                 setattr(geom, k, int(v))
                 mask |= int(name_to_hint.get(k, 0))
             elif k in ASPECT_FIELDS:
@@ -193,7 +168,7 @@ class ClientWindow(GTKClientWindowBase):
                 setattr(geom, field, float(v))
                 mask |= int(name_to_hint.get(k, 0))
         gdk_hints = Gdk.WindowHints(mask)
-        log("apply_geometry_hints(%s) geometry=%s, hints=%s", hints, geom, gdk_hints)
+        metalog("apply_geometry_hints(%s) geometry=%s, hints=%s", hints, geom, gdk_hints)
         self.set_geometry_hints(None, geom, gdk_hints)
 
 
