@@ -566,9 +566,10 @@ class GTKTrayMenuBase(object):
 
     def make_keyboardsyncmenuitem(self):
         def set_keyboard_sync_tooltip():
-            if not self.client.keyboard_helper:
+            kh = self.client.keyboard_helper
+            if not kh:
                 self.keyboard_sync_menuitem.set_tooltip_text("Keyboard support is not loaded")
-            elif self.client.keyboard_helper.keyboard_sync:
+            elif kh.keyboard_sync:
                 self.keyboard_sync_menuitem.set_tooltip_text("Disable keyboard synchronization (prevents spurious key repeats on high latency connections)")
             else:
                 self.keyboard_sync_menuitem.set_tooltip_text("Enable keyboard state synchronization")
@@ -580,9 +581,11 @@ class GTKTrayMenuBase(object):
         self.keyboard_sync_menuitem = self.checkitem("Keyboard Synchronization", keyboard_sync_toggled)
         set_sensitive(self.keyboard_sync_menuitem, False)
         def set_keyboard_sync_menuitem(*args):
-            log("set_keyboard_sync_menuitem%s enabled=%s", args, self.client.keyboard_helper.keyboard_sync)
-            self.keyboard_sync_menuitem.set_active(self.client.keyboard_helper.keyboard_sync)
-            set_sensitive(self.keyboard_sync_menuitem, True)
+            kh = self.client.keyboard_helper
+            set_sensitive(self.keyboard_sync_menuitem, bool(kh))
+            if kh:
+                log("set_keyboard_sync_menuitem%s enabled=%s", args, kh.keyboard_sync)
+            self.keyboard_sync_menuitem.set_active(bool(kh) and bool(kh.keyboard_sync))
             set_keyboard_sync_tooltip()
         self.client.after_handshake(set_keyboard_sync_menuitem)
         return self.keyboard_sync_menuitem
@@ -891,6 +894,12 @@ class GTKTrayMenuBase(object):
                         menu.ignore_events = False
             c.connect("toggled", activate_cb, menu)
             return c
+        webcam = self.menuitem("Webcam", "webcam.png", "Forward webcam pictures to the server", None)
+        if not self.client.webcam_forwarding:
+            webcam.set_tooltip_text("Webcam forwarding is disabled")
+            set_sensitive(webcam, False)
+            return webcam
+            return c
         def start_webcam(*args):
             webcamlog("start_webcam%s", args)
             self.client.start_sending_webcam()
@@ -902,7 +911,6 @@ class GTKTrayMenuBase(object):
         menu.append(on)
         menu.append(off)
 
-        webcam = self.menuitem("Webcam", "webcam.png", "Forward webcam pictures to the server", None)
         webcam.set_submenu(menu)
         def webcam_changed(*args):
             webcamlog("webcam_changed%s webcam_device=%s", args, self.client.webcam_device)
