@@ -6,6 +6,7 @@
 
 # A tray implemented using gtk.StatusIcon
 
+import os
 import sys
 from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, is_gtk3
 gtk = import_gtk()
@@ -20,13 +21,15 @@ if not is_gtk3():
     ORIENTATION[gtk.ORIENTATION_HORIZONTAL] = "HORIZONTAL"
     ORIENTATION[gtk.ORIENTATION_VERTICAL]   = "VERTICAL"
 
-GUESS_GEOMETRY = sys.platform.startswith("win") or sys.platform.startswith("darwin")
+GUESS_GEOMETRY = str(int(sys.platform.startswith("win") or sys.platform.startswith("darwin")))
+GUESS_GEOMETRY = os.environ.get("XPRA_GUESS_ICON_GEOMETRY", GUESS_GEOMETRY)=="1"
+log("tray GUESS_GEOMETRY=%s", GUESS_GEOMETRY)
 
 
 class GTKStatusIconTray(TrayBase):
 
-    def __init__(self, *args):
-        TrayBase.__init__(self, *args)
+    def __init__(self, *args, **kwargs):
+        TrayBase.__init__(self, *args, **kwargs)
         self.tray_widget = gtk.StatusIcon()
         self.tray_widget.set_tooltip_text(self.tooltip or "Xpra")
         self.tray_widget.connect('activate', self.activate_menu)
@@ -91,16 +94,26 @@ class GTKStatusIconTray(TrayBase):
 
     def get_geometry(self):
         assert self.tray_widget
-        ag = self.tray_widget.get_geometry()
-        log("GTKStatusIconTray.get_geometry() %s.get_geometry()=%s", self.tray_widget, ag)
+        #on X11, if we don't have an xid, don't bother querying its geometry,
+        #as this would trigger some ugly GTK warnings we can do nothing about
+        if os.name=="posix" and os.environ.get("DISPLAY") and self.tray_widget.get_x11_window_id()==0:
+            ag = None
+        else:
+            ag = self.tray_widget.get_geometry()
+            log("GTKStatusIconTray.get_geometry() %s.get_geometry()=%s", self.tray_widget, ag)
         if ag is None:
-            #probably win32 or OSX...
+            #probably win32 or OSX, gnome-shell or KDE5..
             self.may_guess()
-            log("GTKStatusIconTray.get_geometry() returning guess: %s", self.geometry_guess)
+            log("GTKStatusIconTray.get_geometry() no geometry value available, returning guess: %s", self.geometry_guess)
             return self.geometry_guess
         #gtk3 adds an extra argument.. at the beginning!
         _, geom, _ = ag[-3:]
-        return geom.x, geom.y, geom.width, geom.height
+        x, y, w, h = geom.x, geom.y, geom.width, geom.height
+        if x==0 and y==0 and w==200 and h==200:
+            #this isn't right, take a better guess, at least for the size:
+            w = 24
+            h = 64
+        return x, y, w, h
 
     def get_size(self):
         s = max(8, min(256, self.tray_widget.get_size()))
@@ -132,7 +145,20 @@ class GTKStatusIconTray(TrayBase):
         h = tray_icon.get_height()
         log("set_icon_from_pixbuf(%s) geometry=%s, icon size=%s", tray_icon, self.get_geometry(), (w, h))
         if tw!=w or th!=h:
-            tray_icon = tray_icon.scale_simple(tw, th, INTERP_HYPER)
+            if tw!=th:
+                #paste the scaled icon in the middle of the rectangle:
+                minsize = min(tw, th)
+                new_icon = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, tw, th)
+                new_icon.fill(0)
+                scaled_w, scaled_h = minsize, minsize
+                if tw==24 and th==64:
+                    #special case for the gnome-shell dimensions - stretch height..
+                    scaled_w, scaled_h = 24, 48
+                tray_icon = tray_icon.scale_simple(scaled_w, scaled_h, INTERP_HYPER)
+                tray_icon.copy_area(0, 0, scaled_w, scaled_h, new_icon, (tw-scaled_w)//2, (th-scaled_h)//2)
+                tray_icon = new_icon
+            else:
+                tray_icon = tray_icon.scale_simple(tw, th, INTERP_HYPER)
         self.tray_widget.set_from_pixbuf(tray_icon)
 
 
