@@ -11,7 +11,7 @@ from xpra.gtk_common.gdk_atoms import (
                 gdk_atom_array_from_gdk_atom_objects    #@UnresolvedImport
                 )
 
-from xpra.clipboard.clipboard_base import ClipboardProtocolHelperBase, CLIPBOARDS, log
+from xpra.clipboard.clipboard_base import ClipboardProtocolHelperBase, log
 
 
 class GDKClipboardProtocolHelper(ClipboardProtocolHelperBase):
@@ -19,12 +19,10 @@ class GDKClipboardProtocolHelper(ClipboardProtocolHelperBase):
         to and from a form suitable for transport over the wire.
     """
 
-    def __init__(self, send_packet_cb, progress_cb=None, clipboards=CLIPBOARDS, filter_res=None):
-        ClipboardProtocolHelperBase.__init__(self, send_packet_cb, progress_cb, clipboards, filter_res)
-
-    def __str__(self):
+    def __repr__(self):
         return "GDKClipboardProtocolHelper"
 
+
     def _do_munge_raw_selection_to_wire(self, target, datatype, dataformat, data):
         if dataformat == 32 and datatype in ("ATOM", "ATOM_PAIR"):
             log("_do_munge_raw_selection_to_wire(%s, %s, %s, %s:%s:%s) using gdk atom code", target, datatype, dataformat, type(data), len(data), list(data))
@@ -46,7 +44,7 @@ class GDKClipboardProtocolHelper(ClipboardProtocolHelperBase):
             import gtk.gdk
             gdk_atoms = [gtk.gdk.atom_intern(a) for a in data]
             atom_array = gdk_atom_array_from_gdk_atom_objects(gdk_atoms)
-            bdata = struct.pack("=" + "Q" * len(atom_array), *atom_array)
+            bdata = struct.pack("@" + "L" * len(atom_array), *atom_array)
             log("_munge_wire_selection_to_raw(%s, %s, %s, %s:%s)=%s=%s=%s", encoding, datatype, dataformat, type(data), len(data or ""), gdk_atoms, atom_array, list(bdata))
             return bdata
         return ClipboardProtocolHelperBase._munge_wire_selection_to_raw(self, encoding, datatype, dataformat, data)
