@@ -7,9 +7,16 @@
 import time
 
 try:
-    memoryview_type = memoryview
+    #python 2.7 onwards have memoryview:
+    assert memoryview
+    def clone_plane(plane):
+        if type(plane)==memoryview:
+            return plane.tobytes()
+        return plane[:]
 except:
-    memoryview_type = None
+    def clone_plane(plane):
+        return plane[:]
+
 
 class ImageWrapper(object):
 
@@ -45,7 +52,7 @@ class ImageWrapper(object):
         return self.x
 
     def get_y(self):
-        return self.x
+        return self.y
 
     def get_width(self):
         return self.width
@@ -96,38 +103,35 @@ class ImageWrapper(object):
         self.pixel_format = pixel_format
 
     def set_pixels(self, pixels):
+        assert not self.freed
         self.pixels = pixels
 
     def allocate_buffer(self, buf_len, free_existing=1):
+        assert not self.freed
         #only defined for XImage wrappers:
         return 0
 
     def restride(self, *args):
+        assert not self.freed
         #not supported by the generic image wrapper:
         return False
 
     def freeze(self):
+        assert not self.freed
         #some wrappers (XShm) need to be told to stop updating the pixel buffer
         return False
 
-    def clone_plane(self, plane):
-        if type(plane)==memoryview_type:
-            return plane.tobytes()
-        return plane[:]
-
     def clone_pixel_data(self):
+        assert not self.freed
         pixels = self.pixels
         planes = self.planes
-        if self.freed:
-            #image has already been freed!
-            return
         assert pixels, "no pixel data to clone"
         if planes == 0:
             #no planes, simple buffer:
-            self.pixels = self.clone_plane(pixels)
+            self.pixels = clone_plane(pixels)
         else:
             assert planes>0
-            self.pixels = [self.clone_plane(pixels[i]) for i in range(planes)]
+            self.pixels = [clone_plane(pixels[i]) for i in range(planes)]
         self.thread_safe = True
         if self.freed:
             #could be a race since this can run threaded
