@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -20,7 +20,7 @@ if sys.version > '3':
 
 #these codecs may well not load because we
 #do not require the libraries to be installed
-NOWARN = ["nvenc4", "nvenc5", "opencl"]
+NOWARN = ["nvenc4", "nvenc5", "nvenc6", "opencl", "opencv"]
 
 SELFTEST = os.environ.get("XPRA_CODEC_SELFTEST", "1")=="1"
 FULL_SELFTEST = os.environ.get("XPRA_CODEC_FULL_SELFTEST", "0")=="1"
@@ -61,7 +61,13 @@ def codec_import_check(name, description, top_module, class_module, *classnames)
                 if SELFTEST and selftest:
                     if name in CODEC_FAIL_SELFTEST:
                         raise ImportError("codec found in fail selftest list")
-                    selftest(FULL_SELFTEST)
+                    try:
+                        selftest(FULL_SELFTEST)
+                    except Exception as e:
+                        log.warn("Warning: %s failed its self test", name)
+                        log.warn(" %s", e)
+                        log("%s failed", selftest, exc_info=True)
+                        continue
                 #log.warn("codec_import_check(%s, ..)=%s" % (name, ic))
                 log(" found %s : %s", name, ic)
                 codecs[name] = ic
@@ -133,8 +139,10 @@ def load_codecs(encoders=True, decoders=True, csc=True):
         codec_import_check("enc_webp", "webp encoder", "xpra.codecs.webp", "xpra.codecs.webp.encode", "compress")
         add_codec_version("enc_webp", "xpra.codecs.webp.encode")
 
-        codec_import_check("enc_vpx", "vpx encoder", "xpra.codecs.vpx", "xpra.codecs.vpx.encoder", "Encoder")
-        add_codec_version("vpx", "xpra.codecs.vpx.decoder")
+        if not sys.platform.startswith("darwin"):
+            #causes crashes with osx shadow servers, disabled in setup.py:
+            codec_import_check("enc_vpx", "vpx encoder", "xpra.codecs.vpx", "xpra.codecs.vpx.encoder", "Encoder")
+            add_codec_version("vpx", "xpra.codecs.vpx.decoder")
 
         codec_import_check("enc_x264", "x264 encoder", "xpra.codecs.enc_x264", "xpra.codecs.enc_x264.encoder", "Encoder")
         add_codec_version("x264", "xpra.codecs.enc_x264.encoder")
@@ -142,10 +150,13 @@ def load_codecs(encoders=True, decoders=True, csc=True):
         codec_import_check("enc_x265", "x265 encoder", "xpra.codecs.enc_x265", "xpra.codecs.enc_x265.encoder", "Encoder")
         add_codec_version("x265", "xpra.codecs.enc_x265.encoder")
 
-        for v in (4, 5):
+        for v in (4, 5, 6):
             codec_import_check("nvenc%s" % v, "nvenc encoder", "xpra.codecs.nvenc%s" % v, "xpra.codecs.nvenc%s.encoder" % v, "Encoder")
             add_codec_version("nvenc%s" % v, "xpra.codecs.nvenc%s.encoder" % v)
 
+        codec_import_check("xvid", "xvid encoder", "xpra.codecs.xvid", "xpra.codecs.xvid.encoder", "Encoder")
+        add_codec_version("xvid", "xpra.codecs.xvid.encoder")
+
     if csc:
         show += list(CSC_CODECS)
         codec_import_check("csc_swscale", "swscale colorspace conversion", "xpra.codecs.csc_swscale", "xpra.codecs.csc_swscale.colorspace_converter", "ColorspaceConverter")
@@ -157,6 +168,12 @@ def load_codecs(encoders=True, decoders=True, csc=True):
         codec_import_check("csc_opencl", "OpenCL colorspace conversion", "xpra.codecs.csc_opencl", "xpra.codecs.csc_opencl.colorspace_converter", "ColorspaceConverter")
         add_codec_version("opencl", "xpra.codecs.csc_opencl.colorspace_converter")
 
+        codec_import_check("csc_libyuv", "libyuv colorspace conversion", "xpra.codecs.csc_libyuv", "xpra.codecs.csc_libyuv.colorspace_converter", "ColorspaceConverter")
+        add_codec_version("libyuv", "xpra.codecs.csc_libyuv.colorspace_converter")
+
+        codec_import_check("csc_opencv", "OpenCV colorspace conversion", "xpra.codecs.csc_opencv", "xpra.codecs.csc_opencv.colorspace_converter", "ColorspaceConverter")
+        add_codec_version("opencv", "xpra.codecs.csc_opencv.colorspace_converter")
+
     if decoders:
         show += list(DECODER_CODECS)
         codec_import_check("dec_pillow", "Pillow decoder", "xpra.codecs.pillow", "xpra.codecs.pillow.decode", "decode")
@@ -206,15 +223,15 @@ def has_codec(name):
     return name in codecs
 
 
-CSC_CODECS = "csc_swscale", "csc_cython", "csc_opencl"
-ENCODER_CODECS = "enc_pillow", "enc_vpx", "enc_webp", "enc_x264", "enc_x265", "nvenc4", "nvenc5"
+CSC_CODECS = "csc_swscale", "csc_cython", "csc_opencl", "csc_libyuv", "csc_opencv"
+ENCODER_CODECS = "enc_pillow", "enc_vpx", "enc_webp", "enc_x264", "enc_x265", "nvenc4", "nvenc5", "nvenc6", "xvid"
 DECODER_CODECS = "dec_pillow", "dec_vpx", "dec_webp", "dec_avcodec2"
 
 ALL_CODECS = tuple(set(CSC_CODECS + ENCODER_CODECS + DECODER_CODECS))
 
 #note: this is just for defining the order of encodings,
 #so we have both core encodings (rgb24/rgb32) and regular encodings (rgb) in here:
-PREFERED_ENCODING_ORDER = ["h264", "vp9", "vp8", "png", "png/P", "png/L", "webp", "rgb", "rgb24", "rgb32", "jpeg", "h265"]
+PREFERED_ENCODING_ORDER = ["h264", "vp9", "vp8", "mpeg4", "png", "png/P", "png/L", "webp", "rgb", "rgb24", "rgb32", "jpeg", "h265"]
 #encoding order for edges (usually one pixel high or wide):
 EDGE_ENCODING_ORDER = ["rgb24", "rgb32", "jpeg", "png", "webp", "png/P", "png/L", "rgb"]
 
@@ -227,6 +244,7 @@ if compression.get_enabled_compressors():
 ENCODINGS_TO_NAME = {
       "h264"    : "H.264",
       "h265"    : "H.265",
+      "mpeg4"   : "MPEG4",
       "vp8"     : "VP8",
       "vp9"     : "VP9",
       "png"     : "PNG (24/32bpp)",
@@ -242,6 +260,7 @@ ENCODINGS_HELP = {
       "h265"    : "H.265 (HEVC) video codec (slow and buggy - do not use!)",
       "vp8"     : "VP8 video codec",
       "vp9"     : "VP9 video codec",
+      "mpeg4"   : "MPEG-4 video codec",
       "png"     : "Portable Network Graphics (lossless, 24bpp or 32bpp for transparency)",
       "png/P"   : "Portable Network Graphics (lossy, 8bpp colour)",
       "png/L"   : "Portable Network Graphics (lossy, 8bpp grayscale)",
@@ -250,7 +269,7 @@ ENCODINGS_HELP = {
       "rgb"     : "Raw RGB pixels, lossless, compressed using %s (24bpp or 32bpp for transparency)" % (" or ".join(compression.get_enabled_compressors())),
       }
 
-HELP_ORDER = ("h264", "h265", "vp8", "vp9", "png", "png/P", "png/L", "webp", "rgb", "jpeg")
+HELP_ORDER = ("h264", "h265", "vp8", "vp9", "mpeg4", "png", "png/P", "png/L", "webp", "rgb", "jpeg")
 
 #those are currently so useless that we don't want the user to select them by mistake
 PROBLEMATIC_ENCODINGS = ("h265", )
@@ -270,10 +289,10 @@ def encoding_help(encoding):
 
 
 def main():
-    from xpra.platform import init, clean
+    from xpra.platform import program_context
     from xpra.log import enable_color
-    try:
-        init("Loader", "Encoding Info")
+    from xpra.util import print_nested_dict, pver
+    with program_context("Loader", "Encoding Info"):
         enable_color()
         verbose = "-v" in sys.argv or "--verbose" in sys.argv
         if verbose:
@@ -312,18 +331,10 @@ def main():
                     print("error getting extra information on %s: %s" % (name, e))
         print("")
         print("codecs versions:")
-        def pver(v):
-            if type(v)==tuple:
-                return ".".join([str(x) for x in v])
-            elif type(v) in (str, unicode) and v.startswith("v"):
-                return v[1:]
-            return str(v)
-        for name in sorted(codec_versions.keys()):
-            version = codec_versions[name]
-            print("* %s : %s" % (name.ljust(20), pver(version)))
-    finally:
-        #this will wait for input on win32:
-        clean()
+        def forcever(v):
+            return pver(v, numsep=".", strsep=".").lstrip("v")
+        print_nested_dict(codec_versions, vformat=forcever)
+
 
 if __name__ == "__main__":
     main()
