@@ -52,6 +52,14 @@ def remove_debug_category(*cat):
         if c in debug_enabled_categories:
             debug_enabled_categories.remove(c)
 
+def is_debug_enabled(category):
+    if "all" in debug_enabled_categories:
+        return True
+    if category in debug_enabled_categories:
+        return True
+    return isenvdebug(category) or isenvdebug("ALL")
+
+
 def add_disabled_category(*cat):
     remove_debug_category(*cat)
     for c in cat:
@@ -94,11 +102,13 @@ def set_default_level(level):
     default_level = level
 
 
-#this allows us to capture all logging and redirect it:
 def standard_logging(log, level, msg, *args, **kwargs):
     #this is just the regular logging:
     log(level, msg, *args, **kwargs)
 
+#this allows us to capture all logging and redirect it:
+#the default 'standard_logging' uses the logger,
+#but the client may inject its own handler here
 global_logging_handler = standard_logging
 
 def set_global_logging_handler(h):
@@ -186,7 +196,9 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ("cuda"         , "CUDA device access (nvenc)"),
                 ("cython"       , "Cython CSC module"),
                 ("opencl"       , "OpenCL CSC module"),
+                ("opencv"       , "OpenCV CSC module"),
                 ("swscale"      , "swscale CSC module"),
+                ("libyuv"       , "libyuv CSC module"),
                 ("decoder"      , "All decoders"),
                 ("encoder"      , "All encoders"),
                 ("avcodec"      , "avcodec decoder"),
@@ -196,7 +208,9 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ("nvenc"        , "nvenc encoder (all versions)"),
                 ("x264"         , "libx264 encoder"),
                 ("x265"         , "libx265 encoder"),
+                ("xvid"         , "Xvid encoder"),
                 ("webp"         , "libwebp encoder and decoder"),
+                ("webcam"       , "webcam access"),
                 ])),
     ("Pointer", OrderedDict([
                 ("mouse"        , "Mouse motion"),
@@ -227,6 +241,7 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ("shadow"       , "Shadow server"),
                 ("command"      , "Server control channel"),
                 ("timeout"      , "Server timeouts"),
+                ("exec"         , "Executing commands"),
                 #server features:
                 ("mdns"         , "mDNS session publishing"),
                 #server internals:
@@ -236,6 +251,7 @@ STRUCT_KNOWN_FILTERS = OrderedDict([
                 ])),
     ("Sound", OrderedDict([
                 ("sound"        , "All sound"),
+                ("gstreamer"    , "GStreamer internal messages"),
                 ("av-sync"      , "Audio-video sync"),
                 ])),
     ("X11", OrderedDict([
@@ -294,7 +310,7 @@ class Logger(object):
         for cat in self.categories:
             if cat in debug_disabled_categories:
                 disabled = True
-            if "all" in debug_enabled_categories or cat in debug_enabled_categories or isenvdebug(cat) or isenvdebug("ALL"):
+            if is_debug_enabled(cat):
                 enabled = True
         self.debug_enabled = enabled and not disabled
         #ready, keep track of it:
