@@ -116,20 +116,32 @@ _COMPRESSORS = {
                }
 
 def get_compression_caps():
-    caps = {
-            "lz4"                   : use_lz4,
-            "lzo"                   : use_lzo,
-            "zlib"                  : use_zlib,
-            "zlib.version"          : zlib.__version__,
-           }
+    caps = {}
+    _lzo = {"" : use_lzo}
     if lzo_version:
-        caps["lzo.version"] = lzo_version
+        _lzo["version"] = lzo_version
     if python_lzo_version:
-        caps["python-lzo.version"] = python_lzo_version
+        caps["python-lzo"] = {
+                              ""            : True,
+                              "version"     : python_lzo_version,
+                              }
+    _lz4 = {""  : use_lz4}
     if lz4_version:
-        caps["lz4.version"] = lz4_version
+        _lz4["version"] = lz4_version
     if python_lz4_version:
-        caps["python-lz4.version"] = python_lz4_version
+        caps["python-lz4"] = {
+                              ""            : True,
+                              "version"     : python_lz4_version,
+                              }
+    _zlib = {
+             ""             : use_zlib,
+             "version"      : zlib.__version__
+             }
+    caps.update({
+                 "lz4"                   : _lz4,
+                 "lzo"                   : _lzo,
+                 "zlib"                  : _zlib,
+                 })
     return caps
 
 def get_enabled_compressors(order=ALL_COMPRESSORS):
@@ -262,14 +274,10 @@ def decompress_by_name(data, algo):
 
 
 def main():
-    from xpra.platform import init, clean
-    try:
-        init("Compression", "Compression Info")
-        for k,v in sorted(get_compression_caps().items()):
-            print(k.ljust(20)+": "+str(v))
-    finally:
-        #this will wait for input on win32:
-        clean()
+    from xpra.util import print_nested_dict
+    from xpra.platform import program_context
+    with program_context("Compression", "Compression Info"):
+        print_nested_dict(get_compression_caps())
 
 
 if __name__ == "__main__":
