@@ -11,9 +11,16 @@ from xpra.log import Logger
 log = Logger("network", "protocol")
 from xpra.net.header import FLAGS_RENCODE, FLAGS_YAML, FLAGS_BENCODE
 
+#those are also modified from the command line switch:
+use_rencode = os.environ.get("XPRA_USE_RENCODER", "1")=="1"
+use_bencode = os.environ.get("XPRA_USE_BENCODER", "1")=="1"
+use_yaml    = os.environ.get("XPRA_USE_YAML", "1")=="1"
 
+
+has_rencode = None
 rencode_dumps, rencode_loads, rencode_version = None, None, None
-try:
+def init_rencode():
+    global use_rencode, has_rencode, rencode_dumps, rencode_loads, rencode_version
     try:
         import rencode
         rencode_dumps = rencode.dumps
@@ -25,40 +32,55 @@ try:
             log.warn("rencode at '%s' lacks versioning information", rencode.__file__)
             rencode_version = "unknown"
     except ImportError as e:
-        log.warn("rencode import error: %s", e)
-except Exception as e:
-    log.error("error loading rencode", exc_info=True)
-has_rencode = rencode_dumps is not None and rencode_loads is not None and rencode_version is not None
-use_rencode = has_rencode and os.environ.get("XPRA_USE_RENCODER", "1")=="1"
-log("packet encoding: has_rencode=%s, use_rencode=%s, version=%s", has_rencode, use_rencode, rencode_version)
+        log("init_rencode()", exc_info=True)
+        if use_rencode:
+            log.warn("Warning: rencode import failed:")
+            log.warn(" %s", e)
+    except Exception as e:
+        log.error("error loading rencode", exc_info=True)
+    has_rencode = rencode_dumps is not None and rencode_loads is not None and rencode_version is not None
+    use_rencode = has_rencode and use_rencode
+    log("packet_encoding.init_rencode() has_rencode=%s, use_rencode=%s, version=%s", has_rencode, use_rencode, rencode_version)
 
 
+has_bencode = None
 bencode, bdecode, bencode_version = None, None, None
-try:
+def init_bencode():
+    global use_bencode, has_bencode, bencode, bdecode, bencode_version
     try:
         from xpra.net.bencode import bencode, bdecode, __version__ as bencode_version
     except ImportError as e:
-        log.warn("bencode import error: %s", e, exc_info=True)
-except Exception as e:
-    log.error("error loading bencoder", exc_info=True)
-has_bencode = bencode is not None and bdecode is not None
-use_bencode = has_bencode and os.environ.get("XPRA_USE_BENCODER", "1")=="1"
-log("packet encoding: has_bencode=%s, use_bencode=%s, version=%s", has_bencode, use_bencode, bencode_version)
-
+        log("init_bencode()", exc_info=True)
+        if use_bencode:
+            log.warn("Warning: bencode import failed:")
+            log.warn(" %s", e)
+    except Exception as e:
+        log.error("error loading bencoder", exc_info=True)
+    has_bencode = bencode is not None and bdecode is not None
+    use_bencode = has_bencode and use_bencode
+    log("packet_encoding.init_bencode() has_bencode=%s, use_bencode=%s, version=%s", has_bencode, use_bencode, bencode_version)
 
+has_yaml = None
 yaml_encode, yaml_decode, yaml_version = None, None, None
-try:
-    #json messes with strings and unicode (makes it unusable for us)
-    import yaml
-    yaml_encode = yaml.dump
-    yaml_decode = yaml.load
-    yaml_version = yaml.__version__
-except ImportError:
-    log("yaml not found")
-has_yaml = yaml_encode is not None and yaml_decode is not None
-use_yaml = has_yaml and os.environ.get("XPRA_USE_YAML", "1")=="1"
-log("packet encoding: has_yaml=%s, use_yaml=%s, version=%s", has_yaml, use_yaml, yaml_version)
+def init_yaml():
+    global use_yaml, has_yaml, yaml_encode, yaml_decode, yaml_version
+    try:
+        #json messes with strings and unicode (makes it unusable for us)
+        import yaml
+        yaml_encode = yaml.dump
+        yaml_decode = yaml.load
+        yaml_version = yaml.__version__
+    except ImportError:
+        log("yaml not found")
+    has_yaml = yaml_encode is not None and yaml_decode is not None
+    use_yaml = has_yaml and use_yaml
+    log("packet encoding: has_yaml=%s, use_yaml=%s, version=%s", has_yaml, use_yaml, yaml_version)
 
+def init():
+    init_rencode()
+    init_bencode()
+    init_yaml()
+init()
 
 def do_bencode(data):
     return bencode(data), FLAGS_BENCODE
@@ -71,21 +93,23 @@ def do_yaml(data):
 
 
 def get_packet_encoding_caps():
-    caps = {
-            "rencode"               : use_rencode,
-            "bencode"               : use_bencode,
-            "yaml"                  : use_yaml,
-           }
+    r = {"" : use_rencode}
     if has_rencode:
         assert rencode_version is not None
-        caps["rencode.version"] = rencode_version
+        r["version"]    = rencode_version
+    b = {"" : use_bencode}
     if has_bencode:
         assert bencode_version is not None
-        caps["bencode.version"] = bencode_version
+        b["version"] = bencode_version
+    y = {"" : use_yaml}
     if has_yaml:
         assert yaml_version is not None
-        caps["yaml.version"] = yaml_version
-    return caps
+        y["version"] = yaml_version
+    return {
+            "rencode"               : r,
+            "bencode"               : b,
+            "yaml"                  : y,
+           }
 
 
 #all the encoders we know about, in best compatibility order:
@@ -95,10 +119,10 @@ ALL_ENCODERS = ["bencode", "rencode", "yaml"]
 PERFORMANCE_ORDER = ["rencode", "bencode", "yaml"]
 
 _ENCODERS = {
-        "rencode"   : do_rencode,
-        "bencode"   : do_bencode,
-        "yaml"      : do_yaml,
-           }
+             "rencode"   : do_rencode,
+             "bencode"   : do_bencode,
+             "yaml"      : do_yaml,
+             }
 
 def get_enabled_encoders(order=ALL_ENCODERS):
     enabled = [x for x,b in {
@@ -167,14 +191,10 @@ def decode(data, protocol_flags):
 
 
 def main():
-    from xpra.platform import init, clean
-    try:
-        init("Packet Encoding", "Packet Encoding Info")
-        for k,v in sorted(get_packet_encoding_caps().items()):
-            print(k.ljust(20)+": "+str(v))
-    finally:
-        #this will wait for input on win32:
-        clean()
+    from xpra.util import print_nested_dict
+    from xpra.platform import program_context
+    with program_context("Packet Encoding", "Packet Encoding Info"):
+        print_nested_dict(get_packet_encoding_caps())
 
 
 if __name__ == "__main__":
