@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2015 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2016 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2009, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -20,7 +20,7 @@ log = Logger("network", "protocol")
 cryptolog = Logger("network", "crypto")
 
 from xpra.os_util import Queue, strtobytes
-from xpra.util import repr_ellipsized, updict, csv
+from xpra.util import repr_ellipsized, csv
 from xpra.net import ConnectionClosedException
 from xpra.net.bytestreams import ABORT
 from xpra.net import compression
@@ -29,7 +29,7 @@ from xpra.net.compression import get_compression_caps, decompress, sanity_checks
         InvalidCompressionException, Compressed, LevelCompressed, Uncompressed
 from xpra.net.packet_encoding import get_packet_encoding_caps, decode, sanity_checks as packet_encoding_sanity_checks, InvalidPacketEncodingException
 from xpra.net.header import unpack_header, pack_header, FLAGS_CIPHER, FLAGS_NOHEADER
-from xpra.net.crypto import get_crypto_caps, get_cipher, pad, INITIAL_PADDING
+from xpra.net.crypto import get_crypto_caps, get_encryptor, get_decryptor, pad, INITIAL_PADDING
 
 
 #stupid python version breakage:
@@ -189,7 +189,7 @@ class Protocol(object):
             cryptolog.info("receiving data using %s encryption", ciphername)
             self.cipher_in_name = ciphername
         cryptolog("set_cipher_in%s", (ciphername, iv, password, key_salt, iterations))
-        self.cipher_in, self.cipher_in_block_size = get_cipher(ciphername, iv, password, key_salt, iterations)
+        self.cipher_in, self.cipher_in_block_size = get_decryptor(ciphername, iv, password, key_salt, iterations)
         self.cipher_in_padding = padding
 
     def set_cipher_out(self, ciphername, iv, password, key_salt, iterations, padding):
@@ -197,7 +197,7 @@ class Protocol(object):
             cryptolog.info("sending data using %s encryption", ciphername)
             self.cipher_out_name = ciphername
         cryptolog("set_cipher_out%s", (ciphername, iv, password, key_salt, iterations, padding))
-        self.cipher_out, self.cipher_out_block_size = get_cipher(ciphername, iv, password, key_salt, iterations)
+        self.cipher_out, self.cipher_out_block_size = get_encryptor(ciphername, iv, password, key_salt, iterations)
         self.cipher_out_padding = padding
 
 
@@ -210,19 +210,26 @@ class Protocol(object):
 
     def get_info(self, alias_info=True):
         info = {
-            "input.packetcount"     : self.input_packetcount,
-            "input.raw_packetcount" : self.input_raw_packetcount,
-            "input.cipher"          : self.cipher_in_name or "",
-            "input.cipher.padding"  : self.cipher_in_padding,
-            "output.packetcount"    : self.output_packetcount,
-            "output.raw_packetcount": self.output_raw_packetcount,
-            "output.cipher"         : self.cipher_out_name or "",
-            "output.cipher.padding" : self.cipher_out_padding,
             "large_packets"         : self.large_packets,
             "compression_level"     : self.compression_level,
-            "max_packet_size"       : self.max_packet_size}
-        updict(info, "input.count", self.input_stats)
-        updict(info, "output.count", self.output_stats)
+            "max_packet_size"       : self.max_packet_size,
+            "input" : {
+                       "packetcount"            : self.input_packetcount,
+                       "raw_packetcount"        : self.input_raw_packetcount,
+                       "count"                  : self.input_stats,
+                       "cipher"                 : {"": self.cipher_in_name or "",
+                                                   "padding"        : self.cipher_in_padding,
+                                                   },
+                        },
+            "output" : {
+                        "packetcount"           : self.output_packetcount,
+                        "raw_packetcount"       : self.output_raw_packetcount,
+                        "count"                 : self.output_stats,
+                        "cipher"                : {"": self.cipher_out_name or "",
+                                                   "padding" : self.cipher_out_padding
+                                                   },
+                        },
+            }
         c = self._compress
         if c:
             info["compressor"] = compression.get_compressor_name(self._compress)
@@ -233,12 +240,8 @@ class Protocol(object):
             else:
                 info["encoder"] = packet_encoding.get_encoder_name(self._encoder)
         if alias_info:
-            for k,v in self.send_aliases.items():
-                info["send_alias." + str(k)] = v
-                info["send_alias." + str(v)] = k
-            for k,v in self.receive_aliases.items():
-                info["receive_alias." + str(k)] = v
-                info["receive_alias." + str(v)] = k
+            info["send_alias"] = self.send_aliases
+            info["receive_alias"] = self.receive_aliases
         c = self._conn
         if c:
             try:
@@ -248,7 +251,7 @@ class Protocol(object):
         info["has_more"] = self._source_has_more.is_set()
         for t in (self._write_thread, self._read_thread, self._read_parser_thread, self._write_format_thread):
             if t:
-                info["thread.%s" % t.name] = t.is_alive()
+                info.setdefault("thread", {})[t.name] = t.is_alive()
         return info
 
 
@@ -551,10 +554,10 @@ class Protocol(object):
         except (OSError, IOError, socket_error) as e:
             if not self._closed:
                 self._internal_error("%s connection %s reset" % (name, self._conn), e, exc_info=e.args[0] not in ABORT)
-        except:
+        except Exception as e:
             #can happen during close(), in which case we just ignore:
             if not self._closed:
-                log.error("%s error on %s", name, self._conn, exc_info=True)
+                log.error("Error: %s on %s failed: %s", name, self._conn, type(e), exc_info=True)
                 self.close()
 
     def _write_thread_loop(self):
@@ -597,7 +600,9 @@ class Protocol(object):
         self._read_queue_put(buf)
         if not buf:
             log("read thread: eof")
-            self.close()
+            #give time to the parse thread to call close itself
+            #so it has time to parse and process the last packet received
+            self.timeout_add(1000, self.close)
             return
         self.input_raw_packetcount += 1
 
@@ -608,7 +613,7 @@ class Protocol(object):
         ei = exc_info
         if exc:
             ei = None   #log it separately below
-        log.error("internal error: %s", message, exc_info=ei)
+        log.error("Error: %s", message, exc_info=ei)
         if exc:
             log.error(" %s", exc, exc_info=exc_info)
         self.idle_add(self._connection_lost, message)
@@ -682,7 +687,7 @@ class Protocol(object):
         while not self._closed:
             buf = self._read_queue.get()
             if not buf:
-                log("read thread: empty marker, exiting")
+                log("parse thread: empty marker, exiting")
                 self.idle_add(self.close)
                 return
             if read_buffer:
