@@ -53,7 +53,7 @@ except:
         from io import StringIO as StringIOClass        #@Reimport @UnusedImport
 
 
-if sys.version < '3':
+if sys.version_info[0] < 3:
     def strtobytes(x):
         return str(x)
     def bytestostr(x):
@@ -91,23 +91,19 @@ def platform_name(sys_platform, release):
                  "os2"      : "OS/2",
                  }
     def rel(v):
-        if sys_platform=="win32" and release:
-            return "%s %s" % (v, release)
-        return v
+        values = [v]
+        if sys_platform=="win32" or sys_platform.startswith("linux"):
+            if type(release) in (tuple, list):
+                values += list(release)
+            else:
+                values.append(release)
+        return " ".join([str(x) for x in values if x])
     for k,v in PLATFORMS.items():
         regexp = re.compile(k)
         if regexp.match(sys_platform):
             return rel(v)
     return rel(sys_platform)
 
-def os_info(sys_platform, platform_release, platform_platform, platform_linux_distribution):
-    s = [platform_name(sys_platform, platform_release)]
-    if platform_linux_distribution and len(platform_linux_distribution)==3 and len(platform_linux_distribution[0])>0:
-        s.append(" ".join([str(x) for x in platform_linux_distribution]))
-    elif platform_platform:
-        s.append(platform_platform)
-    return s
-
 
 def get_hex_uuid():
     return uuid.uuid4().hex
@@ -155,6 +151,32 @@ def is_Ubuntu():
     except:
         return False
 
+def is_Fedora():
+    try:
+        assert os.name=="posix"
+        v = load_binary_file("/etc/issue")
+        return v.find("Fedora")>=0
+    except:
+        return False
+
+def getUbuntuVersion():
+    from xpra.scripts.config import python_platform
+    distro = ""
+    if hasattr(python_platform, "linux_distribution"):
+        distro = python_platform.linux_distribution()
+    if distro and len(distro)==3 and distro[0]=="Ubuntu":
+        ur = distro[1]  #ie: "12.04"
+        try:
+            rnum = [int(x) for x in ur.split(".")]  #ie: [12, 4]
+            return rnum
+        except:
+            pass
+    return []
+
+def is_unity():
+    return os.environ.get("XDG_CURRENT_DESKTOP", "").lower() == "unity"
+
+
 def load_binary_file(filename):
     if not os.path.exists(filename):
         return None
@@ -169,6 +191,60 @@ def force_quit(status=1):
     os._exit(status)
 
 
+def livefds():
+    live = set()
+    try:
+        MAXFD = os.sysconf("SC_OPEN_MAX")
+    except:
+        MAXFD = 256
+    for fd in range(0, MAXFD):
+        try:
+            s = os.fstat(fd)
+            if s:
+                live.add(fd)
+        except:
+            continue
+    return live
+
+#code to temporarily redirect stderr and restore it afterwards, adapted from:
+#http://stackoverflow.com/questions/5081657/how-do-i-prevent-a-c-shared-library-to-print-on-stdout-in-python
+#used by the sound code to get rid of the stupid gst warning below:
+#"** Message: pygobject_register_sinkfunc is deprecated (GstObject)"
+#ideally we would redirect to a buffer so we could still capture and show these messages in debug out
+class HideStdErr(object):
+
+    def __init__(self, *args, **kw):
+        self.savedstderr = None
+
+    def __enter__(self):
+        if os.name=="posix" and os.getppid()==1:
+            #this interferes with server daemonizing?
+            return
+        sys.stderr.flush() # <--- important when redirecting to files
+        self.savedstderr = os.dup(2)
+        devnull = os.open(os.devnull, os.O_WRONLY)
+        os.dup2(devnull, 2)
+        os.close(devnull)
+        sys.stderr = os.fdopen(self.savedstderr, 'w')
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if self.savedstderr is not None:
+            os.dup2(self.savedstderr, 2)
+
+class HideSysArgv(object):
+
+    def __init__(self, *args, **kw):
+        self.savedsysargv = None
+
+    def __enter__(self):
+        self.savedsysargv = sys.argv
+        sys.argv = sys.argv[:1]
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if self.savedsysargv is not None:
+            sys.argv = self.savedsysargv
+
+
 def disable_stdout_buffering():
     import gc
     # Appending to gc.garbage is a way to stop an object from being
