@@ -8,7 +8,7 @@ import os
 from xpra.log import Logger
 log = Logger("osx", "events")
 
-SLEEP_HANDLER = os.environ.get("XPRA_OSX_SLEEP_HANDLER", "0")=="1"
+SLEEP_HANDLER = os.environ.get("XPRA_OSX_SLEEP_HANDLER", "1")=="1"
 
 
 exit_cb = None
@@ -127,6 +127,49 @@ def get_window_frame_size(x, y, w, h):
                }
 
 
+def get_icc_info():
+    from AppKit import NSScreen     #@UnresolvedImport
+    ms = NSScreen.mainScreen()
+    info = do_get_screen_icc_info(ms)
+    screens = NSScreen.screens()
+    for i, screen in enumerate(screens):
+        si = do_get_screen_icc_info(screen)
+        if si:
+            info[i] = si
+    return info
+
+def do_get_screen_icc_info(nsscreen):
+    info = {}
+    try:
+        from AppKit import NSUnknownColorSpaceModel, NSGrayColorSpaceModel, NSRGBColorSpaceModel, NSCMYKColorSpaceModel, NSLABColorSpaceModel, NSDeviceNColorSpaceModel, NSIndexedColorSpaceModel, NSPatternColorSpaceModel    #@UnresolvedImport
+        COLORSPACE_STR = {
+                          NSUnknownColorSpaceModel  : "unknown",
+                          NSGrayColorSpaceModel     : "gray",
+                          NSRGBColorSpaceModel      : "RGB",
+                          NSCMYKColorSpaceModel     : "CMYK",
+                          NSLABColorSpaceModel      : "LAB",
+                          NSDeviceNColorSpaceModel  : "DeviceN",
+                          NSIndexedColorSpaceModel  : "Indexed",
+                          NSPatternColorSpaceModel  : "Pattern",
+                          }
+        try:
+            mscs = nsscreen.colorSpace()
+        except AttributeError as e:
+            #OSX 10.5.x and earlier don't have this attribute
+            return info
+        log("%s.colorSpace=%s", nsscreen, mscs)
+        info.update({
+                     "colorspace"   : COLORSPACE_STR.get(mscs.colorSpaceModel(), "unknown"),
+                     "data"         : str(mscs.ICCProfileData() or ""),
+                     "name"         : str(mscs.localizedName()),
+                     "components"   : mscs.numberOfColorComponents(),
+                     })
+    except Exception as e:
+        log.warn("Warning: cannot query ICC profiles:")
+        log.warn(" %s", e)
+    return info
+
+
 #global menu handling:
 window_menus = {}
 
@@ -215,9 +258,6 @@ def window_focused(window, event):
 
 def add_window_hooks(window):
     window.connect("focus-in-event", window_focused)
-    client = getattr(window, "_client", None)
-    if client:
-        init_dock_listener(client)
 
 def remove_window_hooks(window):
     try:
@@ -256,11 +296,19 @@ except:
     CG = None
 
 
-def get_CG_imagewrapper():
+def roundup(n, m):
+    return (n + m - 1) & ~(m - 1)
+
+def get_CG_imagewrapper(rect=None):
     from xpra.codecs.image_wrapper import ImageWrapper
     assert CG, "cannot capture without Quartz.CoreGraphics"
-    #region = CG.CGRectMake(0, 0, 100, 100)
-    region = CG.CGRectInfinite
+    if rect is None:
+        x = 0
+        y = 0
+        region = CG.CGRectInfinite
+    else:
+        x, y, w, h = rect
+        region = CG.CGRectMake(x, y, roundup(w, 2), roundup(h, 2))
     image = CG.CGWindowListCreateImage(region,
                 CG.kCGWindowListOptionOnScreenOnly,
                 CG.kCGNullWindowID,
@@ -275,7 +323,7 @@ def get_CG_imagewrapper():
     log("get_CG_imagewrapper(..) image size: %sx%s, bpc=%s, bpp=%s, rowstride=%s, alpha=%s", width, height, bpc, bpp, rowstride, alpha_str)
     prov = CG.CGImageGetDataProvider(image)
     argb = CG.CGDataProviderCopyData(prov)
-    return ImageWrapper(0, 0, width, height, argb, "BGRX", 24, rowstride)
+    return ImageWrapper(x, y, width, height, argb, "BGRX", 24, rowstride)
 
 def take_screenshot():
     log("grabbing screenshot")
@@ -293,62 +341,13 @@ def take_screenshot():
 
 
 try:
-    import AppKit                               #@UnresolvedImport
-    import PyObjCTools.AppHelper as AppHelper   #@UnresolvedImport
-    from AppKit import NSObject, NSApplication, NSWorkspace #@UnresolvedImport
+    from AppKit import NSObject                 #@UnresolvedImport
 except Exception as e:
     log.warn("Warning: failed to load critical modules")
     log.warn(" %s", e)
     log.warn(" cannot enable sleep notification support")
-    log.warn(" and dock click notification")
+    log.warn(" or dock click notification")
     NSObject = object
-    NSApplication = None
-    NSWorkspace = None
-
-
-class NotificationHandler(NSObject):
-    """Class that handles the sleep notifications."""
-
-    def handleSleepNotification_(self, aNotification):
-        log("handleSleepNotification(%s) sleep_callback=%s", aNotification, self.sleep_callback)
-        if self.sleep_callback:
-            try:
-                self.sleep_callback()
-            except:
-                log.error("Error in sleep callback %s", self.sleep_callback, exc_info=True)
-
-    def handleWakeNotification_(self, aNotification):
-        log("handleWakeNotification(%s) wake_callback=%s", aNotification, self.wake_callback)
-        if self.wake_callback:
-            try:
-                self.wake_callback()
-            except:
-                log.error("Error in wake callback %s", self.wake_callback, exc_info=True)
-
-
-dock_listener = None
-def init_dock_listener(client):
-    global dock_listener
-    if dock_listener:
-        return
-    try:
-        import objc         #@UnresolvedImport
-        shared_app = NSApplication.sharedApplication()
-
-        class Delegate(NSObject):
-            @objc.signature('B@:#B')
-            def applicationShouldHandleReopen_hasVisibleWindows_(self, ns_app, flag):
-                log("applicationShouldHandleReopen_hasVisibleWindows%s", (ns_app, flag))
-                client.deiconify_windows()
-                return True
-
-        delegate = Delegate.alloc().init()
-        delegate.retain()
-        shared_app.setDelegate_(delegate)
-        dock_listener = delegate
-    except:
-        log.error("Error setting up dock listener", exc_info=True)
-        dock_listener = False
 
 
 class ClientExtras(object):
@@ -356,10 +355,7 @@ class ClientExtras(object):
         swap_keys = opts and opts.swap_keys
         log("ClientExtras.__init__(%s, %s) swap_keys=%s", client, opts, swap_keys)
         self.client = client
-        self.notificationCenter = None
-        self.handler = None
-        if SLEEP_HANDLER:
-            self.setup_event_loop()
+        self.event_loop_started = False
         if opts and client:
             log("setting swap_keys=%s using %s", swap_keys, client.keyboard_helper)
             if client.keyboard_helper and client.keyboard_helper.keyboard:
@@ -368,57 +364,95 @@ class ClientExtras(object):
 
     def cleanup(self):
         self.client = None
-        self.stop_event_loop()
 
-    def stop_event_loop(self):
-        if self.notificationCenter:
-            self.notificationCenter = None
-        if self.handler:
-            self.handler = None
+    def ready(self):
+        try:
+            self.setup_event_listener()
+        except:
+            log.error("Error setting up OSX event listener", exc_info=True)
 
-    def setup_event_loop(self):
-        if NSWorkspace is None:
-            self.notificationCenter = None
-            self.handler = None
-            log.warn("event loop not started")
+    def setup_event_listener(self):
+        log("setup_event_listener()")
+        if NSObject is object:
+            log.warn("NSObject is missing, not setting up OSX event listener")
             return
-        ws = NSWorkspace.sharedWorkspace()
-        self.notificationCenter = ws.notificationCenter()
-        self.handler = NotificationHandler.new()
+        self.shared_app = None
+        self.delegate = None
+        from AppKit import NSApplication, NSWorkspace, NSWorkspaceWillSleepNotification, NSWorkspaceDidWakeNotification     #@UnresolvedImport
+        import objc         #@UnresolvedImport
+        self.shared_app = NSApplication.sharedApplication()
+
+        class Delegate(NSObject):
+            def applicationDidFinishLaunching_(self, notification):
+                log("applicationDidFinishLaunching_(%s)", notification)
+                if SLEEP_HANDLER:
+                    self.register_sleep_handlers()
+
+            def register_sleep_handlers(self):
+                log("register_sleep_handlers()")
+                workspace          = NSWorkspace.sharedWorkspace()
+                notificationCenter = workspace.notificationCenter()
+                notificationCenter.addObserver_selector_name_object_(self, self.receiveSleepNotification_,
+                                                                     NSWorkspaceWillSleepNotification, None)
+                notificationCenter.addObserver_selector_name_object_(self, self.receiveWakeNotification_,
+                                                                     NSWorkspaceDidWakeNotification, None)
+
+            @objc.signature('B@:#B')
+            def applicationShouldHandleReopen_hasVisibleWindows_(self, ns_app, flag):
+                log("applicationShouldHandleReopen_hasVisibleWindows%s", (ns_app, flag))
+                self.cb("deiconify_callback")
+                return True
+
+            def receiveSleepNotification_(self, aNotification):
+                log("receiveSleepNotification_(%s) sleep_callback=%s", aNotification, self.sleep_callback)
+                self.cb("sleep_callback")
+
+            def receiveWakeNotification_(self, aNotification):
+                log("receiveWakeNotification_(%s)", aNotification)
+                self.cb("wake_callback")
+
+            def cb(self, name):
+                #find the named callback and call it
+                callback = getattr(self, name, None)
+                log("cb(%s)=%s", name, callback)
+                if callback:
+                    try:
+                        callback()
+                    except:
+                        log.error("Error in %s callback %s", name, callback, exc_info=True)
+
+        self.delegate = Delegate.alloc().init()
+        self.delegate.retain()
         if self.client:
-            self.handler.sleep_callback = self.client.suspend
-            self.handler.wake_callback = self.client.resume
-        else:
-            self.handler.sleep_callback = None
-            self.handler.wake_callback = None
-        self.notificationCenter.addObserver_selector_name_object_(
-                 self.handler, "handleSleepNotification:",
-                 AppKit.NSWorkspaceWillSleepNotification, None)
-        self.notificationCenter.addObserver_selector_name_object_(
-                 self.handler, "handleWakeNotification:",
-                 AppKit.NSWorkspaceDidWakeNotification, None)
-        log("starting console event loop with notifcation center=%s and handler=%s", self.notificationCenter, self.handler)
+            self.delegate.sleep_callback = self.client.suspend
+            self.delegate.wake_callback = self.client.resume
+            self.delegate.deiconify_callback = self.client.deiconify_windows
+        self.shared_app.setDelegate_(self.delegate)
+        log("setup_event_listener() the application delegate has been registered")
 
 
     def run(self):
         #this is for running standalone
+        log("starting console event loop")
+        self.event_loop_started = True
+        import PyObjCTools.AppHelper as AppHelper   #@UnresolvedImport
         AppHelper.runConsoleEventLoop(installInterrupt=True)
         #when running from the GTK main loop, we rely on another part of the code
         #to run the event loop for us
 
     def stop(self):
-        AppHelper.stopEventLoop()
+        if self.event_loop_started:
+            self.event_loop_started = False
+            import PyObjCTools.AppHelper as AppHelper   #@UnresolvedImport
+            AppHelper.stopEventLoop()
 
 
 def main():
-    from xpra.platform import init, clean
-    try:
-        init("OSX Extras")
+    from xpra.platform import program_context
+    with program_context("OSX Extras"):
         log.enable_debug()
         ce = ClientExtras(None, None)
         ce.run()
-    finally:
-        clean()
 
 
 if __name__ == "__main__":
