@@ -1,3 +1,4 @@
+#!/usr/bin/env python
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
 # Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
@@ -77,6 +78,47 @@ def get_icon_size():
 def get_antialias_info():
     return {}
 
+def get_icc_info():
+    from xpra.log import Logger
+    log = Logger("platform")
+    info = {}
+    try:
+        from PIL.ImageCms import get_display_profile, \
+            getProfileName, getProfileInfo, getProfileCopyright, getProfileManufacturer, getProfileModel, getProfileDescription, getDefaultIntent, \
+            INTENT_PERCEPTUAL, INTENT_RELATIVE_COLORIMETRIC, INTENT_SATURATION, INTENT_ABSOLUTE_COLORIMETRIC
+        INTENT_STR = {
+                      INTENT_PERCEPTUAL             : "perceptual",
+                      INTENT_RELATIVE_COLORIMETRIC  : "relative-colorimetric",
+                      INTENT_SATURATION             : "saturation",
+                      INTENT_ABSOLUTE_COLORIMETRIC  : "absolute-colorimetric",
+                      }
+        def getDefaultIntentStr(_p):
+            return INTENT_STR.get(getDefaultIntent(_p), "unknown")
+        def getData(_p):
+            return _p.tobytes()
+        p = get_display_profile()
+        if p:
+            for (k, fn) in {
+                            "name"          : getProfileName,
+                            "info"          : getProfileInfo,
+                            "copyright"     : getProfileCopyright,
+                            "manufacturer"  : getProfileManufacturer,
+                            "model"         : getProfileModel,
+                            "description"   : getProfileDescription,
+                            "default-intent": getDefaultIntentStr,
+                            "data"          : getData,
+                            }.items():
+                try:
+                    v = fn(p)
+                    info[k] = v
+                except Exception as e:
+                    log("ICC profile error on %s using %s: %s", k, fn, e)
+    except Exception as e:
+        log.warn("Warning: cannot query ICC profiles:")
+        log.warn(" %s", e)
+    return info
+
+
 #global workarea for all screens
 def get_workarea():
     return None
@@ -94,6 +136,9 @@ def get_desktop_names():
 def get_vrefresh():
     return -1
 
+def get_mouse_config():
+    return {}
+
 def get_double_click_time():
     return -1
 
@@ -136,6 +181,9 @@ def gl_check():
 def get_menu_support_function():
     return None
 
+def get_wm_name():
+    return None
+
 
 take_screenshot = None
 ClientExtras = None
@@ -149,29 +197,34 @@ def get_info_base():
             return str(v)
     def fnames(l):
         return [fname(x) for x in l]
-    info = {
+    return {
             "native_tray_menu_helpers"      : fnames(get_native_tray_menu_helper_classes()),
             "native_trays"                  : fnames(get_native_tray_classes()),
             "native_system_trays"           : fnames(get_native_system_tray_classes()),
             "system_bell"                   : fname(system_bell),
             "native_notifiers"              : fnames(get_native_notifier_classes()),
+            "wm_name"                       : get_wm_name() or "",
             "workarea"                      : get_workarea() or "",
             "workareas"                     : get_workareas(),
             "desktops"                      : get_number_of_desktops(),
             "desktop_names"                 : get_desktop_names(),
             "vertical-refresh"              : get_vrefresh(),
-            "double_click.time"             : get_double_click_time(),
-            "double_click.distance"         : get_double_click_distance(),
             "fixed_cursor_size"             : get_fixed_cursor_size(),
             "cursor_size"                   : get_cursor_size(),
-            "dpi.x"                         : get_xdpi(),
-            "dpi.y"                         : get_ydpi(),
             "icon_size"                     : get_icon_size(),
+            "mouse"                         : get_mouse_config(),
+            "double_click"                  : {
+                                               "time"       : get_double_click_time(),
+                                               "distance"   : get_double_click_distance(),
+                                               },
+            "dpi"                           : {
+                                               "x"          : get_xdpi(),
+                                               "y"          : get_ydpi(),
+                                               },
+            "antialias"                     : get_antialias_info(),
+            "icc"                           : get_icc_info(),
+            "window_frame"                  : get_window_frame_sizes(),
             }
-    from xpra.util import updict
-    updict(info, "antialias", get_antialias_info())
-    updict(info, "window_frame", get_window_frame_sizes())
-    return info
 
 get_info = get_info_base
 
@@ -181,6 +234,7 @@ platform_import(globals(), "gui", False,
                 "do_ready",
                 "do_init",
                 "gl_check",
+                "get_wm_name",
                 "show_desktop", "set_fullscreen_monitors", "set_shaded",
                 "ClientExtras",
                 "take_screenshot",
@@ -191,7 +245,9 @@ platform_import(globals(), "gui", False,
                 "get_native_notifier_classes",
                 "get_vrefresh", "get_workarea", "get_workareas",
                 "get_number_of_desktops", "get_desktop_names",
-                "get_antialias_info", "get_icon_size", "get_xdpi", "get_ydpi",
+                "get_antialias_info", "get_icc_info", "get_xdpi", "get_ydpi",
+                "get_icon_size",
+                "get_mouse_config",
                 "get_double_click_time", "get_double_click_distance",
                 "get_fixed_cursor_size", "get_cursor_size", "get_window_frame_sizes",
                 "add_window_hooks", "remove_window_hooks",
@@ -200,11 +256,10 @@ platform_import(globals(), "gui", False,
 
 
 def main():
-    from xpra.platform import init as platform_init,clean
-    from xpra.util import nonl
+    from xpra.platform import program_context
+    from xpra.util import print_nested_dict
     from xpra.log import enable_color
-    try:
-        platform_init("GUI-Properties")
+    with program_context("GUI-Properties"):
         enable_color()
         init()
         verbose = "-v" in sys.argv or "--verbose" in sys.argv
@@ -221,11 +276,7 @@ def main():
             except:
                 pass    #maybe running on OSX? hope for the best..
         i = get_info()
-        for k in sorted(i.keys()):
-            v = i[k]
-            print("* %s : %s" % (k.ljust(32), nonl(v)))
-    finally:
-        clean()
+        print_nested_dict(i)
 
 
 if __name__ == "__main__":
