@@ -1,3 +1,4 @@
+#!/usr/bin/env python
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
 # Copyright (C) 2011-2015 Antoine Martin <antoine@devloop.org.uk>
@@ -77,10 +78,15 @@ def do_get_download_dir():
         return "~"
     return d
 
-def get_script_bin_dir():
-    return env_or_delegate("XPRA_SCRIPT_BIN_DIR", do_get_script_bin_dir)
-def do_get_script_bin_dir():
-    return "~/.xpra"
+def get_script_bin_dirs():
+    return envaslist_or_delegate("XPRA_SCRIPT_BIN_DIRS", do_get_script_bin_dirs)
+def do_get_script_bin_dirs():
+    return ["~/.xpra"]
+
+def get_remote_run_xpra_scripts():
+    return envaslist_or_delegate("XPRA_REMOTE_RUN_XPRA_SCRIPTS", do_get_remote_run_xpra_scripts)
+def do_get_remote_run_xpra_scripts():
+    return ["~/.xpra/run-xpra", "$XDG_RUNTIME_DIR/xpra/run-xpra", "xpra"]
 
 
 #overriden in platform code:
@@ -196,19 +202,19 @@ platform_import(globals(), "paths", False,
                 "do_get_socket_dirs",
                 "do_get_default_log_dir",
                 "do_get_download_dir",
-                "do_get_script_bin_dir")
+                "do_get_script_bin_dirs")
 
 def get_info():
     return {
-            "install.prefix"    : get_install_prefix(),
-            "default_conf.dirs" : get_default_conf_dirs(),
-            "system_conf.dirs"  : get_system_conf_dirs(),
-            "user_conf.dirs"    : get_user_conf_dirs(),
-            "socket.dirs"       : get_socket_dirs(),
-            "log.dir"           : get_default_log_dir(),
-            "download.dir"      : get_download_dir(),
-            "app.dir"           : get_app_dir(),
-            "app.default.dir"   : default_get_app_dir(),
+            "install"           : {"prefix" : get_install_prefix()},
+            "default_conf"      : {"dirs"   : get_default_conf_dirs()},
+            "system_conf"       : {"dirs"   : get_system_conf_dirs()},
+            "user_conf"         : {"dirs"   : get_user_conf_dirs()},
+            "socket"            : {"dirs"   : get_socket_dirs()},
+            "log"               : {"dir"    : get_default_log_dir()},
+            "download"          : {"dir"    : get_download_dir()},
+            "app"               : {"dir"    : get_app_dir()},
+            "app"               : {"default" : {"dir"   : default_get_app_dir()}},
             "resources"         : get_resources_dir(),
             "icons"             : get_icon_dir(),
             "home"              : os.path.expanduser("~"),
@@ -221,17 +227,11 @@ def main():
         from xpra.log import add_debug_category
         add_debug_category("util")
 
-    from xpra.util import nonl, pver
-    def print_dict(d):
-        for k in sorted(d.keys()):
-            v = d[k]
-            print("* %s : %s" % (k.ljust(32), nonl(pver(v))))
-    from xpra.platform import init, clean
-    try:
-        init("Path-Info", "Path Info")
-        print_dict(get_info())
-    finally:
-        clean()
+    from xpra.util import print_nested_dict
+    from xpra.platform import program_context
+    with program_context("Path-Info", "Path Info"):
+        print_nested_dict(get_info())
+
 
 if __name__ == "__main__":
     main()
