@@ -8,11 +8,12 @@
 import sys
 import os
 import cups
+import time
 import subprocess
 import shlex
 import getpass
 import urllib
-
+from threading import Lock
 
 from xpra.util import engs
 from xpra.log import Logger
@@ -107,16 +108,33 @@ def get_lpinfo_drv(make_and_model):
     def preexec():
         os.setsid()
     log("get_lpinfo_drv(%s) command=%s", make_and_model, command)
-    proc = subprocess.Popen(command, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False, close_fds=True, preexec_fn=preexec)
+    try:
+        proc = subprocess.Popen(command, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False, close_fds=True, preexec_fn=preexec)
+    except Exception as e:
+        log("get_lp_info_drv(%s) lpinfo command %s failed", make_and_model, command, exc_info=True)
+        log.error("Error: lpinfo command failed to run")
+        log.error(" %s", e)
+        log.error(" command used: '%s'", " ".join(command))
+        return None
     #use the global child reaper to make sure this doesn't end up as a zombie
     from xpra.child_reaper import getChildReaper
     from xpra.util import nonl
     cr = getChildReaper()
     cr.add_process(proc, "lpinfo", command, ignore=True, forget=True)
+    from xpra.make_thread import make_thread
+    def watch_lpinfo():
+        #give it 15 seconds to run:
+        for _ in range(15):
+            if proc.poll() is not None:
+                return      #finished already
+            time.sleep(1)
+        log.warn("Warning: lpinfo command is taking too long")
+        proc.terminate()
+    make_thread(watch_lpinfo, "lpinfo watcher", daemon=True).start()
     out, err = proc.communicate()
     if proc.wait()!=0:
         log.warn("Warning: lpinfo command failed and returned %s", proc.returncode)
-        log.warn(" command used: '%s'", command)
+        log.warn(" command used: '%s'", " ".join(command))
         return None
     if sys.version_info[0]>=3:
         try:
@@ -149,44 +167,46 @@ def add_printer_def(mimetype, definition):
 
 
 PRINTER_DEF = None
+PRINTER_DEF_LOCK = Lock()
 def get_printer_definitions():
-    global PRINTER_DEF, UNPROBED_PRINTER_DEFS, MIMETYPE_TO_PRINTER
-    if PRINTER_DEF is not None:
-        return PRINTER_DEF
-    log("get_printer_definitions() UNPROBED_PRINTER_DEFS=%s, GENERIC=%s", UNPROBED_PRINTER_DEFS, GENERIC)
-    from xpra.platform.printing import get_mimetypes
-    mimetypes = get_mimetypes()
-    #first add the user-supplied definitions:
-    PRINTER_DEF = UNPROBED_PRINTER_DEFS.copy()
-    #raw mode if supported:
-    if RAW_MODE:
-        PRINTER_DEF["raw"] = ["-o", "raw"]
-    #now probe for generic printers via lpinfo:
-    if GENERIC:
+    global PRINTER_DEF, PRINTER_DEF_LOCK, UNPROBED_PRINTER_DEFS, MIMETYPE_TO_PRINTER
+    with PRINTER_DEF_LOCK:
+        if PRINTER_DEF is not None:
+            return PRINTER_DEF
+        log("get_printer_definitions() UNPROBED_PRINTER_DEFS=%s, GENERIC=%s", UNPROBED_PRINTER_DEFS, GENERIC)
+        from xpra.platform.printing import get_mimetypes
+        mimetypes = get_mimetypes()
+        #first add the user-supplied definitions:
+        PRINTER_DEF = UNPROBED_PRINTER_DEFS.copy()
+        #raw mode if supported:
+        if RAW_MODE:
+            PRINTER_DEF["raw"] = ["-o", "raw"]
+        #now probe for generic printers via lpinfo:
+        if GENERIC:
+            for mt in mimetypes:
+                if mt in PRINTER_DEF:
+                    continue    #we have a pre-defined one already
+                x = MIMETYPE_TO_PRINTER.get(mt)
+                if not x:
+                    log.warn("Warning: unknown mimetype '%s', cannot find printer definition", mt)
+                    continue
+                drv = get_lpinfo_drv(x)
+                if drv:
+                    #ie: ["-m", "drv:///sample.drv/generic.ppd"]
+                    PRINTER_DEF[mt] = ["-m", drv]
+        #fallback to locating ppd files:
         for mt in mimetypes:
             if mt in PRINTER_DEF:
-                continue    #we have a pre-defined one already
-            x = MIMETYPE_TO_PRINTER.get(mt)
+                continue        #we have a generic or pre-defined one already
+            x = MIMETYPE_TO_PPD.get(mt)
             if not x:
-                log.warn("Warning: unknown mimetype '%s', cannot find printer definition", mt)
+                log.warn("Warning: unknown mimetype '%s', cannot find corresponding PPD file", mt)
                 continue
-            drv = get_lpinfo_drv(x)
-            if drv:
-                #ie: ["-m", "drv:///sample.drv/generic.ppd"]
-                PRINTER_DEF[mt] = ["-m", drv]
-    #fallback to locating ppd files:
-    for mt in mimetypes:
-        if mt in PRINTER_DEF:
-            continue        #we have a generic or pre-defined one already
-        x = MIMETYPE_TO_PPD.get(mt)
-        if not x:
-            log.warn("Warning: unknown mimetype '%s', cannot find corresponding PPD file", mt)
-            continue
-        f = find_ppd_file(mt.replace("application/", "").upper(), x)
-        if f:
-            #ie: ["-P", "/usr/share/cups/model/Generic-PDF_Printer-PDF.ppd"]
-            PRINTER_DEF[mt] = ["-P", f]
-    log("pycups settings: PRINTER_DEF=%s", PRINTER_DEF)
+            f = find_ppd_file(mt.replace("application/", "").upper(), x)
+            if f:
+                #ie: ["-P", "/usr/share/cups/model/Generic-PDF_Printer-PDF.ppd"]
+                PRINTER_DEF[mt] = ["-P", f]
+        log("pycups settings: PRINTER_DEF=%s", PRINTER_DEF)
     return PRINTER_DEF
 
 
@@ -370,7 +390,7 @@ def print_files(printer, filenames, title, options):
         raise Exception("invalid printer: '%s'" % printer)
     log("pycups.print_files%s", (printer, filenames, title, options))
     actual_options = DEFAULT_CUPS_OPTIONS.copy()
-    actual_options.update(options)
+    actual_options.update(dict((str(k),str(v)) for k,v in options.items()))
     if SIMULATE_PRINT_FAILURE:
         log.warn("Warning: simulating print failure")
         conn = None
@@ -405,6 +425,33 @@ PRINTER_STATE = {
                  }
 
 
+def get_info():
+    from xpra.platform.printing import get_mimetypes, DEFAULT_MIMETYPES
+    return {"mimetypes"         : {""           : get_mimetypes(),
+                                   "default"    : DEFAULT_MIMETYPES,
+                                   "printers"   : MIMETYPE_TO_PRINTER,
+                                   "ppd"        : MIMETYPE_TO_PPD},
+            "mimetype"          : {"default"    : DEFAULT_MIMETYPE},
+            "simulate-failure"  : SIMULATE_PRINT_FAILURE,
+            "allow-user"        : ALLOW,
+            "raw-mode"          : RAW_MODE,
+            "generic"           : GENERIC,
+            "tmpdir"            : FORWARDER_TMPDIR,
+            "lpadmin"           : LPADMIN,
+            "lpinfo"            : LPINFO,
+            "forwarder"         : FORWARDER_BACKEND,
+            "skipped-printers"  : SKIPPED_PRINTERS,
+            "add-local-printers": ADD_LOCAL_PRINTERS,
+            "printer-prefix"    : PRINTER_PREFIX,
+            "cups-dbus"         : {""           : CUPS_DBUS,
+                                   "default"    : DEFAULT_CUPS_DBUS,
+                                   "poll-delay" : POLLING_DELAY},
+            "cups.default-options"  : DEFAULT_CUPS_OPTIONS,
+            "printers"          : {""           : get_printer_definitions(),
+                                   "predefined" : UNPROBED_PRINTER_DEFS},
+            }
+
+
 def main():
     if "-v" in sys.argv or "--verbose" in sys.argv:
         from xpra.log import add_debug_category, enable_debug_for
@@ -419,10 +466,9 @@ def main():
         except:
             pass
 
-    from xpra.platform import init, clean
+    from xpra.platform import program_context
     from xpra.log import enable_color
-    try:
-        init("PyCUPS Printing")
+    with program_context("PyCUPS Printing"):
         enable_color()
         validate_setup()
         log.info("")
@@ -438,8 +484,6 @@ def main():
                 if pk=="printer-state" and pv in PRINTER_STATE:
                     pv = "%s (%s)" % (pv, PRINTER_STATE.get(pv))
                 log.info("    %-32s: %s", pk, pv)
-    finally:
-        clean()
 
 
 if __name__ == "__main__":
