@@ -5,8 +5,9 @@
 
 import os
 import time
+import threading
 from threading import Event
-from xpra.daemon_thread import make_daemon_thread
+from xpra.make_thread import make_thread
 from xpra.log import Logger
 log = Logger("util")
 
@@ -51,15 +52,17 @@ class UI_thread_watcher(object):
         #run once to initialize:
         self.UI_thread_wakeup()
         if self.polling_timeout>0:
-            make_daemon_thread(self.poll_UI_loop, "UI thread polling").start()
+            make_thread(self.poll_UI_loop, "UI thread polling").start()
         else:
             log("not starting an IO polling thread")
         if FAKE_UI_LOCKUPS>0:
             #watch out: sleeping in UI thread!
             def sleep_in_ui_thread(*args):
-                time.sleep(FAKE_UI_LOCKUPS)
+                t = threading.current_thread()
+                log.warn("sleep_in_ui_thread%s pausing %s for %ims", args, t, FAKE_UI_LOCKUPS)
+                time.sleep(FAKE_UI_LOCKUPS/1000.0)
                 return True
-            self.timeout_add((10+FAKE_UI_LOCKUPS)*1000, sleep_in_ui_thread)
+            self.timeout_add(10*1000+FAKE_UI_LOCKUPS, sleep_in_ui_thread)
 
     def stop(self):
         self.exit.set()
@@ -106,8 +109,17 @@ class UI_thread_watcher(object):
                 log("poll_UI_loop() ok, firing %s", self.alive_callbacks)
                 self.run_callbacks(self.alive_callbacks)
             self.timeout_add(0, self.UI_thread_wakeup)
-            self.exit.wait(self.polling_timeout/1000.0)
+            wstart = time.time()
+            wait_time = self.polling_timeout/1000.0     #convert to seconds
+            self.exit.wait(wait_time)
+            if not self.exit.isSet():
+                wdelta = time.time() - wstart
+                log("wait(%.4f) actually waited %.4f", self.polling_timeout/1000.0, wdelta)
+                if wdelta>(wait_time+1):
+                    log.warn("Warning: long timer waiting time,")
+                    log.warn(" UI thread polling waited %.1f seconds longer than intended (%.1f vs %.1f)", wdelta-wait_time, wdelta, wait_time)
         self.init_vars()
+        log("poll_UI_loop() ended")
 
 
 UI_watcher = None
@@ -115,4 +127,5 @@ def get_UI_watcher(timeout_add=None):
     global UI_watcher
     if UI_watcher is None and timeout_add:
         UI_watcher = UI_thread_watcher(timeout_add, POLLING)
+        log("get_UI_watcher(%s)", timeout_add)
     return UI_watcher
