@@ -10,9 +10,14 @@ import errno
 import os.path
 import sys
 
+import win32con         #@UnresolvedImport
+import win32api         #@UnresolvedImport
+import win32console     #@UnresolvedImport
+
 #redirect output if we are launched from py2exe's gui mode:
-REDIRECT_OUTPUT = hasattr(sys, "frozen") and sys.frozen=="windows_exe"
-if getattr(sys, 'frozen', False):
+frozen = getattr(sys, 'frozen', False)
+REDIRECT_OUTPUT = frozen=="windows_exe"
+if frozen:
     #cx_freeze paths:
     def jedir(relpathname):
         return os.path.join(os.path.dirname(sys.executable), relpathname)
@@ -26,12 +31,11 @@ if getattr(sys, 'frozen', False):
     addsyspath('bin\\lib')
     addsyspath('bin\\share')
     addsyspath('bin\\library.zip')
-    os.environ['GI_TYPELIB_PATH'] = jedir('bin\\lib\girepository-1.0')
+    os.environ['GI_TYPELIB_PATH'] = jedir('lib\\girepository-1.0')
 
 
 def is_wine():
     try:
-        import win32con, win32api     #@UnresolvedImport
         hKey = win32api.RegOpenKey(win32con.HKEY_LOCAL_MACHINE, r"Software\\Wine")
         return hKey is not None
     except:
@@ -45,7 +49,6 @@ def set_prgname(name):
     global prg_name
     prg_name = name
     try:
-        import win32api                     #@UnresolvedImport
         win32api.SetConsoleTitle(name)
         import glib
         glib.set_prgname(name)
@@ -216,7 +219,6 @@ def set_wait_for_input():
         _wait_for_input = False
         return
     try:
-        import win32console, win32con       #@UnresolvedImport
         handle = win32console.GetStdHandle(win32console.STD_OUTPUT_HANDLE)
         #handle.SetConsoleTextAttribute(win32console.FOREGROUND_BLUE)
         console_info = handle.GetConsoleScreenBufferInfo()
@@ -256,6 +258,41 @@ def do_init():
     sys.stderr = sys.stdout
 
 
+CONSOLE_EXIT_EVENTS = []
+CONSOLE_EXIT_EVENTS = [win32con.CTRL_C_EVENT,
+                       win32con.CTRL_LOGOFF_EVENT,
+                       win32con.CTRL_BREAK_EVENT,
+                       win32con.CTRL_SHUTDOWN_EVENT,
+                       win32con.CTRL_CLOSE_EVENT]
+class console_event_catcher(object):
+    def __init__(self, event_cb, events=CONSOLE_EXIT_EVENTS):
+        self.event_cb = event_cb
+        self.events = events
+        self.result = 0
+        from xpra.log import Logger
+        self.log = Logger("win32")
+    def __enter__(self):
+        try:
+            self.result = win32api.SetConsoleCtrlHandler(self.handle_console_event, 1)
+            if self.result == 0:
+                self.log.error("could not SetConsoleCtrlHandler (error %r)", win32api.GetLastError())
+        except Exception as e:
+            self.log.error("SetConsoleCtrlHandler error: %s", e)
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        try:
+            win32api.SetConsoleCtrlHandler(None, 0)
+        except:
+            pass
+    def __repr__(self):
+        return "console_event_catcher(%s, %s)" % (self.event_cb, self.events)
+
+    def handle_console_event(self, event):
+        self.log("handle_console_event(%s)", event)
+        if event in self.events:
+            self.log.info("received console event %s", event)
+            self.event_cb(event)
+
+
 SHOW_MESSAGEBOX = os.environ.get("XPRA_MESSAGEBOX", "1")=="1"
 MB_ICONEXCLAMATION  = 0x00000030
 MB_ICONINFORMATION  = 0x00000040
@@ -267,7 +304,6 @@ def _show_message(message, uType):
     if SHOW_MESSAGEBOX and GUI_MODE:
         #try to use an alert box since no console output will be shown:
         try:
-            import win32api         #@UnresolvedImport
             win32api.MessageBox(0, message, prg_name, uType)
             return
         except:
