@@ -13,6 +13,8 @@ from xpra.log import Logger
 log = Logger("win32")
 grablog = Logger("win32", "grab")
 screenlog = Logger("win32", "screen")
+keylog = Logger("win32", "keyboard")
+mouselog = Logger("win32", "mouse")
 
 from xpra.platform.win32.win32_events import get_win32_event_listener
 from xpra.platform.win32.window_hooks import Win32Hooks
@@ -20,6 +22,10 @@ from xpra.util import AdHocStruct, csv
 import ctypes
 from ctypes import windll, byref
 
+import win32con     #@UnresolvedImport
+import win32api     #@UnresolvedImport
+import win32gui     #@UnresolvedImport
+
 WINDOW_HOOKS = os.environ.get("XPRA_WIN32_WINDOW_HOOKS", "1")=="1"
 GROUP_LEADER = WINDOW_HOOKS and os.environ.get("XPRA_WIN32_GROUP_LEADER", "1")=="1"
 UNDECORATED_STYLE = WINDOW_HOOKS and os.environ.get("XPRA_WIN32_UNDECORATED_STYLE", "1")=="1"
@@ -31,12 +37,15 @@ LANGCHANGE = WINDOW_HOOKS and os.environ.get("XPRA_WIN32_LANGCHANGE", "1")=="1"
 
 DPI_AWARE = os.environ.get("XPRA_DPI_AWARE", "1")=="1"
 DPI_AWARENESS = int(os.environ.get("XPRA_DPI_AWARENESS", "1"))
+FORWARD_WINDOWS_KEY = os.environ.get("XPRA_FORWARD_WINDOWS_KEY", "0")=="1"
+WHEEL = os.environ.get("XPRA_WHEEL", "1")=="1"
+WHEEL_DELTA = int(os.environ.get("XPRA_WHEEL_DELTA", "120"))
+assert WHEEL_DELTA>0
 
 
 KNOWN_EVENTS = {}
 POWER_EVENTS = {}
 try:
-    import win32con             #@UnresolvedImport
     for x in dir(win32con):
         if x.endswith("_EVENT"):
             v = getattr(win32con, x)
@@ -44,13 +53,15 @@ try:
         if x.startswith("PBT_"):
             v = getattr(win32con, x)
             POWER_EVENTS[v] = x
-    import win32api             #@UnresolvedImport
-    import win32gui             #@UnresolvedImport
 except Exception as e:
     log.warn("error loading pywin32: %s", e)
 
 
 def do_init():
+    init_dpi()
+
+
+def init_dpi():
     #tell win32 we handle dpi
     if not DPI_AWARE:
         screenlog.warn("SetProcessDPIAware not set due to environment override")
@@ -309,7 +320,19 @@ def add_window_hooks(window):
     except:
         handle = None
     if not handle:
-        log.warn("cannot add window hooks without a window handle!")
+        from xpra.gtk_common.gobject_compat import is_gtk3
+        if is_gtk3():
+            #access the missing gdk_win32_window_get_handle function using ctypes:
+            try:
+                ctypes.pythonapi.PyCapsule_GetPointer.restype = ctypes.c_void_p
+                ctypes.pythonapi.PyCapsule_GetPointer.argtypes = [ctypes.py_object]
+                gdkwin_gpointer = ctypes.pythonapi.PyCapsule_GetPointer(gdk_window.__gpointer__, None)
+                gdkdll = ctypes.CDLL ("libgdk-3-0.dll")
+                handle = gdkdll.gdk_win32_window_get_handle(gdkwin_gpointer)
+            except Exception as e:
+                log.warn("failed to get window handle", exc_info=True)
+    if not handle:
+        log.warn("Warning: cannot add window hooks without a window handle!")
         return
     log("add_window_hooks(%s) gdk window=%s, hwnd=%#x", window, gdk_window, handle)
 
@@ -328,7 +351,7 @@ def add_window_hooks(window):
             window.fixup_window_style = types.MethodType(fixup_window_style, window)
             #override set_decorated so we can preserve the taskbar menu for undecorated windows
             window.__set_decorated = window.set_decorated
-            window.set_decorated = types.MethodType(set_decorated, window, type(window))
+            window.set_decorated = types.MethodType(set_decorated, window)
             #override after_window_state_updated so we can re-add the missing style options
             #(somehow doing it from on_realize which calls add_window_hooks is not enough)
             window.connect("state-updated", window_state_updated)
@@ -357,6 +380,58 @@ def add_window_hooks(window):
                 log("WM_INPUTLANGCHANGE: character set: %i, input locale identifier: %i", wParam, lParam)
                 window.keyboard_layout_changed("WM_INPUTLANGCHANGE", wParam, lParam)
             win32hooks.add_window_event_handler(win32con.WM_INPUTLANGCHANGE, inputlangchange)
+        if WHEEL:
+            VERTICAL = "vertical"
+            HORIZONTAL = "horizontal"
+            class WheelEvent(AdHocStruct):
+                pass
+            def handle_wheel(orientation, wParam, lParam):
+                distance = wParam>>16
+                keys = wParam & 0xFFFF
+                y = lParam>>16
+                x = lParam & 0xFFFF
+                #FIXME: we should reset those values when the window loses focus..
+                cval = getattr(window, "_win32_%swheel" % orientation, 0)
+                nval = cval + distance
+                units = nval // WHEEL_DELTA
+                client = getattr(window, "_client")
+                wid = getattr(window, "_id", 0)
+                mouselog("mousewheel: orientation=%s distance=%.1f, units=%i, new value=%.1f, keys=%#x, x=%i, y=%i, client=%s, wid=%i", orientation, distance, units, nval, keys, x, y, client, wid)
+                if units!=0 and client and wid>0:
+                    if orientation==VERTICAL:
+                        button = 4 + int(units<0)       #4 for UP, 5 for DOWN
+                    else:
+                        button = 7 - int(units<0)       #6 for LEFT, 7 for RIGHT
+                    buttons = []
+                    modifiers = client.get_current_modifiers()
+                    def send_button(pressed):
+                        client.send_button(wid, button, pressed, (x, y), modifiers, buttons)
+                    count = 0
+                    v = nval
+                    while abs(v)>=WHEEL_DELTA:
+                        send_button(True)
+                        send_button(False)
+                        if v>0:
+                            v -= WHEEL_DELTA
+                        else:
+                            v += WHEEL_DELTA
+                        count += 1
+                    mouselog("mousewheel: send %i wheel events to the server for distance=%s, remainder=%s", count, nval, v)
+                    setattr(window, "_win32_%swheel" % orientation, v)
+            def mousewheel(hwnd, event, wParam, lParam):
+                handle_wheel(VERTICAL, wParam, lParam)
+                return 0
+            def mousehwheel(hwnd, event, wParam, lParam):
+                handle_wheel(HORIZONTAL, wParam, lParam)
+                return 0
+            WM_MOUSEHWHEEL = 0x020E
+            win32hooks.add_window_event_handler(win32con.WM_MOUSEWHEEL, mousewheel)
+            win32hooks.add_window_event_handler(WM_MOUSEHWHEEL, mousehwheel)
+            def reset_wheel_counters(*args):
+                mouselog("window lost focus, resetting current wheel deltas")
+                for orientation in (VERTICAL, HORIZONTAL):
+                    setattr(window, "_win32_%swheel" % orientation, 0)
+            window.connect("focus-out-event", reset_wheel_counters)
 
 
 def remove_window_hooks(window):
@@ -406,28 +481,64 @@ FE_FONTSMOOTHING_STR = {
     FE_FONTSMOOTHINGCLEARTYPE   : "ClearType",
     }
 
+
+def _add_SPI(info, constant, name, convert, default=None):
+    SystemParametersInfo = windll.user32.SystemParametersInfoA
+    i = ctypes.c_uint32()
+    if SystemParametersInfo(constant, 0, byref(i), 0):
+        info[name] = convert(i.value)
+    elif default is not None:
+        info[name] = default
+
 def get_antialias_info():
     info = {}
     try:
-        SystemParametersInfo = windll.user32.SystemParametersInfoA
-        def add_param(constant, name, convert):
-            i = ctypes.c_uint32()
-            if SystemParametersInfo(constant, 0, byref(i), 0):
-                info[name] = convert(i.value)
-        add_param(SPI_GETFONTSMOOTHING, "enabled", bool)
+        _add_SPI(info, SPI_GETFONTSMOOTHING, "enabled", bool)
         #"Valid contrast values are from 1000 to 2200. The default value is 1400."
-        add_param(SPI_GETFONTSMOOTHINGCONTRAST, "contrast", int)
+        _add_SPI(info, SPI_GETFONTSMOOTHINGCONTRAST, "contrast", int)
         def orientation(v):
             return FE_ORIENTATION_STR.get(v, "unknown")
-        add_param(SPI_GETFONTSMOOTHINGORIENTATION, "orientation", orientation)
+        _add_SPI(info, SPI_GETFONTSMOOTHINGORIENTATION, "orientation", orientation)
         def smoothing_type(v):
             return FE_FONTSMOOTHING_STR.get(v & FE_FONTSMOOTHINGCLEARTYPE, "unknown")
-        add_param(SPI_GETFONTSMOOTHINGTYPE, "type", smoothing_type)
-        add_param(SPI_GETFONTSMOOTHINGTYPE, "hinting", lambda v : bool(v & 0x2))
+        _add_SPI(info, SPI_GETFONTSMOOTHINGTYPE, "type", smoothing_type)
+        _add_SPI(info, SPI_GETFONTSMOOTHINGTYPE, "hinting", lambda v : bool(v & 0x2))
     except Exception as e:
         log.warn("failed to query antialias info: %s", e)
     return info
 
+def get_mouse_config():
+    #not all are present in win32con?
+    SM_CMOUSEBUTTONS = 43
+    SM_CXDRAG = 68
+    SM_CYDRAG = 69
+    SM_MOUSEPRESENT = 19
+    SM_MOUSEHORIZONTALWHEELPRESENT = 91
+    SM_SWAPBUTTON = 23
+    SM_MOUSEWHEELPRESENT = 75
+    wheel_info = {
+                  "vertical"   : win32api.GetSystemMetrics(SM_MOUSEWHEELPRESENT),
+                  "horizontal" : win32api.GetSystemMetrics(SM_MOUSEHORIZONTALWHEELPRESENT),
+                  }
+    SPI_GETWHEELSCROLLLINES = 104
+    SPI_GETWHEELSCROLLCHARS = 0x006C
+    SPI_GETMOUSEVANISH = 4128
+    #rate for each direction:
+    _add_SPI(wheel_info, SPI_GETWHEELSCROLLLINES, "lines", int, 3)
+    _add_SPI(wheel_info, SPI_GETWHEELSCROLLCHARS, "chars", int, 3)
+    info = {
+            "present"       : bool(win32api.GetSystemMetrics(SM_MOUSEPRESENT)),
+            "wheel"         : wheel_info,
+            "buttons"       : win32api.GetSystemMetrics(SM_CMOUSEBUTTONS),
+            "swap"          : bool(win32api.GetSystemMetrics(SM_SWAPBUTTON)),
+            "drag"          : {
+                               "x"  : win32api.GetSystemMetrics(SM_CXDRAG),
+                               "y"  : win32api.GetSystemMetrics(SM_CYDRAG),
+                               }
+            }
+    _add_SPI(info, SPI_GETMOUSEVANISH, "vanish", bool, False)
+    return info
+
 def get_workarea():
     #this is for x11 servers which can only use a single workarea,
     #calculate the total area:
@@ -640,7 +751,6 @@ class ClientExtras(object):
         self._kh_warning = False
         self._console_handler_registered = self.setup_console_event_listener(True)
         try:
-            import win32con                 #@Reimport @UnresolvedImport
             el = get_win32_event_listener(True)
             if el:
                 el.add_event_callback(win32con.WM_ACTIVATEAPP,      self.activateapp)
@@ -651,6 +761,13 @@ class ClientExtras(object):
                 el.add_event_callback(win32con.WM_WININICHANGE,     self.inichange)
         except Exception as e:
             log.error("cannot register focus and power callbacks: %s", e)
+        self.keyboard_hook_id = None
+        if FORWARD_WINDOWS_KEY:
+            from xpra.make_thread import make_thread
+            make_thread(self.init_keyboard_listener, "keyboard-listener", daemon=True).start()
+
+    def ready(self):
+        pass
 
     def cleanup(self):
         log("ClientExtras.cleanup()")
@@ -660,9 +777,102 @@ class ClientExtras(object):
         el = get_win32_event_listener(False)
         if el:
             el.cleanup()
+        khid = self.keyboard_hook_id
+        if khid:
+            self.keyboard_hook_id = None
+            windll.user32.UnhookWindowsHookEx(khid)
         log("ClientExtras.cleanup() ended")
         #self.client = None
 
+    def init_keyboard_listener(self):
+        from ctypes import CFUNCTYPE, c_int, POINTER, Structure
+        from ctypes.wintypes import DWORD, WPARAM, LPARAM
+        class WindowsKeyEvent(AdHocStruct):
+            pass
+        class KBDLLHOOKSTRUCT(Structure):
+            _fields_ = [("vk_code", DWORD),
+                        ("scan_code", DWORD),
+                        ("flags", DWORD),
+                        ("time", c_int),]
+        DOWN = [win32con.WM_KEYDOWN, win32con.WM_SYSKEYDOWN]
+        #UP = [win32con.WM_KEYUP, win32con.WM_SYSKEYUP]
+        ALL_KEY_EVENTS = {win32con.WM_KEYDOWN       : "KEYDOWN",
+                          win32con.WM_SYSKEYDOWN    : "SYSKEYDOWN",
+                          win32con.WM_KEYUP         : "KEYUP",
+                          win32con.WM_SYSKEYUP      : "SYSKEYUP",
+                          }
+        def low_level_keyboard_handler(nCode, wParam, lParam):
+            try:
+                scan_code = lParam.contents.scan_code
+                focused = self.client._focused
+                keyname = {
+                           win32con.VK_LWIN   : "Super_L",
+                           win32con.VK_RWIN   : "Super_R",
+                           }.get(scan_code)
+                keycode = 0
+                modifiers = []
+                kh = self.client.keyboard_helper
+                if focused and keyname and kh and kh.keyboard and wParam in ALL_KEY_EVENTS:
+                    modifier_keycodes = kh.keyboard.modifier_keycodes
+                    modifier_keys = kh.keyboard.modifier_keys
+                    #find the keycode: (try the exact key we hit first)
+                    for x in (keyname, "Super_L", "Super_R"):
+                        keycodes = modifier_keycodes.get(x, [])
+                        for k in keycodes:
+                            #only interested in numeric keycodes:
+                            try:
+                                keycode = int(k)
+                                break
+                            except:
+                                pass
+                        if keycode>0:
+                            break
+                    for vk, modkeynames in {
+                                        win32con.VK_NUMLOCK     : ["Num_Lock"],
+                                        win32con.VK_CAPITAL     : ["Caps_Lock"],
+                                        win32con.VK_CONTROL     : ["Control_L", "Control_R"],
+                                        win32con.VK_SHIFT       : ["Shift_L", "Shift_R"],
+                                        }.items():
+                        if win32api.GetKeyState(vk):
+                            for modkeyname in modkeynames:
+                                mod = modifier_keys.get(modkeyname)
+                                if mod:
+                                    modifiers.append(mod)
+                                    break
+                    #keylog.info("keyboard helper=%s, modifier keycodes=%s", kh, modifier_keycodes)
+                    keylog("scan_code=%s, event=%s, keyname=%s, keycode=%s, modifiers=%s, focused=%s", scan_code, ALL_KEY_EVENTS.get(wParam), keyname, keycode, modifiers, focused)
+                    if keycode>0:
+                        key_event = WindowsKeyEvent()
+                        key_event.keyname = keyname
+                        key_event.pressed = wParam in DOWN
+                        key_event.modifiers = modifiers
+                        key_event.keyval = scan_code
+                        key_event.keycode = keycode
+                        key_event.string = ""
+                        key_event.group = 0
+                        keylog("detected windows key, sending %s", key_event)
+                        self.client.keyboard_helper.send_key_action(focused, key_event)
+                        #swallow this event:
+                        return 1
+            except Exception as e:
+                keylog.error("Error: low level keyboard hook failed")
+                keylog.error(" %s", e)
+            return windll.user32.CallNextHookEx(keyboard_hook_id, nCode, wParam, lParam)
+        # Our low level handler signature.
+        CMPFUNC = CFUNCTYPE(c_int, WPARAM, LPARAM, POINTER(KBDLLHOOKSTRUCT))
+        # Convert the Python handler into C pointer.
+        pointer = CMPFUNC(low_level_keyboard_handler)
+        # Hook both key up and key down events for common keys (non-system).
+        keyboard_hook_id = windll.user32.SetWindowsHookExA(win32con.WH_KEYBOARD_LL, pointer, win32api.GetModuleHandle(None), 0)
+        # Register to remove the hook when the interpreter exits:
+        keylog("init_keyboard_listener() hook_id=%#x", keyboard_hook_id)
+        while True:
+            msg = win32gui.GetMessage(None, 0, 0)
+            keylog("init_keyboard_listener: GetMessage()=%s", msg)
+            win32gui.TranslateMessage(byref(msg))
+            win32gui.DispatchMessage(byref(msg))
+
+
     def wm_move(self, wParam, lParam):
         c = self.client
         log("WM_MOVE: %s/%s client=%s", wParam, lParam, c)
@@ -675,9 +885,18 @@ class ClientExtras(object):
         event_name = WTS_SESSION_EVENTS.get(event, event)
         log("WM_WTSSESSION_CHANGE: %s on session %#x", event_name, session)
         c = self.client
-        if c and event in (WTS_SESSION_LOGOFF, WTS_SESSION_LOCK):
+        if not c:
+            return
+        if event in (WTS_SESSION_LOGOFF, WTS_SESSION_LOCK):
             log("will freeze all the windows")
             c.freeze()
+        elif event in (WTS_SESSION_LOGON, WTS_SESSION_UNLOCK):
+            log("will unfreeze all the windows")
+            #don't unfreeze directly from here,
+            #as the system may not be fully usable yet (see #997)
+            from xpra.gtk_common.gobject_compat import import_glib
+            glib = import_glib()
+            glib.idle_add(c.unfreeze)
 
 
     def inputlangchange(self, wParam, lParam):
@@ -759,9 +978,8 @@ class ClientExtras(object):
 
 
 def main():
-    from xpra.platform import init, clean
-    try:
-        init("Platform-Events", "Platform Events Test")
+    from xpra.platform import program_context
+    with program_context("Platform-Events", "Platform Events Test"):
         if "-v" in sys.argv or "--verbose" in sys.argv:
             from xpra.platform.win32.win32_events import log as win32_event_logger
             log.enable_debug()
@@ -791,9 +1009,7 @@ def main():
             loop.run()
         except KeyboardInterrupt:
             log.info("exiting on keyboard interrupt")
-    finally:
-        #this will wait for input on win32:
-        clean()
+
 
 if __name__ == "__main__":
     main()
