@@ -108,157 +108,158 @@ MOD_KEYS = {
 #FIXME: support native untranslated win32 values,
 # either using a special capability,
 # or by adding a new argument to keypress packets
-VIRTUAL_KEYS = {
+VIRTUAL_KEYS = [
     #"Mappable" codes
-    "EXECUTE"               : "Execute",
-    "EXSEL"                 : "Ex",
+    ("EXECUTE",                 "Execute"),
+    ("EXSEL",                   "Ex"),
     #no useful mapping:
-    "ICO_CLEAR"             : "IcoClr",
-    "ICO_HELP"              : "Help",
-    "DIVIDE"                : "KP_Divide",
-    "MULTIPLY"              : "KP_Multiply",
-    "SUBTRACT"              : "KP_Substract",
-    "ADD"                   : "KP_Add",
-    "NONAME"                : "NoSymbol",
-    "NUMPAD0"               : "KP_0",
-    "NUMPAD1"               : "KP_1",
-    "NUMPAD2"               : "KP_2",
-    "NUMPAD3"               : "KP_3",
-    "NUMPAD4"               : "KP_4",
-    "NUMPAD5"               : "KP_5",
-    "NUMPAD6"               : "KP_6",
-    "NUMPAD7"               : "KP_7",
-    "NUMPAD8"               : "KP_8",
-    "NUMPAD9"               : "KP_9",
-    "OEM_PERIOD"            : "KP_Decimal",
-    "OEM_1"                 : "OEM_1",
-    "OEM_102"               : "OEM_102",
-    "OEM_2"                 : "OEM_2",
-    "OEM_3"                 : "OEM_3",
-    "OEM_4"                 : "OEM_4",
-    "OEM_5"                 : "OEM_5",
-    "OEM_6"                 : "OEM_6",
-    "OEM_7"                 : "OEM_7",
-    "OEM_8"                 : "OEM_8",
-    "OEM_ATTN"              : "Oem",
-    "OEM_AUTO"              : "Auto",
-    "OEM_AX"                : "Ax",
-    #"OEM_BACKTAB"           : "BackSpace",
-    "BACK"                  : "BackSpace",
-    "ESCAPE"                : "Escape",
-    "OEM_CLEAR"             : "OemClr",
-    "OEM_COMMA"             : "OEM_COMMA",
-    "OEM_COPY"              : "Copy",
-    "OEM_CUSEL"             : "Cu",
-    "OEM_ENLW"              : "Enlw",
-    "OEM_FINISH"            : "Finish",
-    "OEM_FJ_LOYA"           : "Loya",
-    "OEM_FJ_MASSHOU"        : "Mashu",
-    "OEM_FJ_ROYA"           : "Roya",
-    "OEM_FJ_TOUROKU"        : "Touroku",
-    "OEM_JUMP"              : "Jump",
-    "OEM_MINUS"             : "OEM_MINUS",
-    "OEM_PA1"               : "OemPa1",
-    "OEM_PA2"               : "OemPa2",
-    "OEM_PA3"               : "OemPa3",
-    #"OEM_PLUS"              : "equal",
-    "OEM_RESET"             : "Reset",
-    "OEM_WSCTRL"            : "WsCtrl",
-    "PA1"                   : "Pa1",
-    #"OEM_102"               : "backslash",
+    ("ICO_CLEAR",               "IcoClr"),
+    ("ICO_HELP",                "Help"),
+    ("DIVIDE",                  "KP_Divide"),
+    ("MULTIPLY",                "KP_Multiply"),
+    ("SUBTRACT",                "KP_Subtract"),
+    ("ADD",                     "KP_Add"),
+    ("NONAME",                  "NoSymbol"),
+    ("RETURN",                  "KP_Enter"),
+    ("NUMPAD0",                 "KP_0"),
+    ("NUMPAD1",                 "KP_1"),
+    ("NUMPAD2",                 "KP_2"),
+    ("NUMPAD3",                 "KP_3"),
+    ("NUMPAD4",                 "KP_4"),
+    ("NUMPAD5",                 "KP_5"),
+    ("NUMPAD6",                 "KP_6"),
+    ("NUMPAD7",                 "KP_7"),
+    ("NUMPAD8",                 "KP_8"),
+    ("NUMPAD9",                 "KP_9"),
+    ("OEM_PERIOD",              "KP_Decimal"),
+    ("OEM_1",                   "OEM_1"),
+    ("OEM_102",                 "OEM_102"),
+    ("OEM_2",                   "OEM_2"),
+    ("OEM_3",                   "OEM_3"),
+    ("OEM_4",                   "OEM_4"),
+    ("OEM_5",                   "OEM_5"),
+    ("OEM_6",                   "OEM_6"),
+    ("OEM_7",                   "OEM_7"),
+    ("OEM_8",                   "OEM_8"),
+    ("OEM_ATTN",                "Oem"),
+    ("OEM_AUTO",                "Auto"),
+    ("OEM_AX",                  "Ax"),
+    #("OEM_BACKTAB",             "BackSpace"),
+    ("BACK",                    "BackSpace"),
+    ("ESCAPE",                  "Escape"),
+    ("OEM_CLEAR",               "OemClr"),
+    ("OEM_COMMA",               "OEM_COMMA"),
+    ("OEM_COPY",                "Copy"),
+    ("OEM_CUSEL",               "Cu"),
+    ("OEM_ENLW",                "Enlw"),
+    ("OEM_FINISH",              "Finish"),
+    ("OEM_FJ_LOYA",             "Loya"),
+    ("OEM_FJ_MASSHOU",          "Mashu"),
+    ("OEM_FJ_ROYA",             "Roya"),
+    ("OEM_FJ_TOUROKU",          "Touroku"),
+    ("OEM_JUMP",                "Jump"),
+    ("OEM_MINUS",               "OEM_MINUS"),
+    ("OEM_PA1",                 "OemPa1"),
+    ("OEM_PA2",                 "OemPa2"),
+    ("OEM_PA3",                 "OemPa3"),
+    #("OEM_PLUS"              : "equal"),
+    ("OEM_RESET",               "Reset"),
+    ("OEM_WSCTRL",              "WsCtrl"),
+    ("PA1",                     "Pa1"),
+    #("OEM_102"               : "backslash"),
     #missing:?
-    #"PACKET"                : "Packet",
-    "PLAY"                  : "Play",
-    "PROCESSKEY"            : "Process",
-    "RETURN"                : "Return",
-    "SELECT"                : "Select",
-    "SEPARATOR"             : "Separator",
-    "SPACE"                 : "Space",
-    "SPACE"                 : "space",
-    "SUBTRACT"              : "minus",
-    "TAB"                   : "Tab",
-    "ZOOM"                  : "Zoom",
+    #("PACKET"                : "Packet"),
+    ("PLAY",                    "Play"),
+    ("PROCESSKEY",              "Process"),
+    ("RETURN",                  "Return"),
+    ("SELECT",                  "Select"),
+    ("SEPARATOR",               "Separator"),
+    ("SPACE",                   "Space"),
+    ("SPACE",                   "space"),
+    ("SUBTRACT",                "minus"),
+    ("TAB",                     "Tab"),
+    ("ZOOM",                    "Zoom"),
     #"Non-mappable" codes
-    "BROWSER_FAVORITES"     : "XF86Favorites",
-    "BROWSER_FORWARD"       : "XF86Forward",
-    "BROWSER_HOME"          : "XF86HomePage",
-    "BROWSER_REFRESH"       : "XF86Reload",
-    "BROWSER_SEARCH"        : "XF86Search",
-    "BROWSER_STOP"          : "XF86Suspend",
-    "CAPITAL"               : "Caps_Lock",
-    "CONVERT"               : "Convert",
-    "DELETE"                : "Delete",
-    "DOWN"                  : "Down",
-    "END"                   : "End",
-    "F1"                    : "F1",
-    "F10"                   : "F10",
-    "F11"                   : "F11",
-    "F12"                   : "F12",
-    "F13"                   : "F13",
-    "F14"                   : "F14",
-    "F15"                   : "F15",
-    "F16"                   : "F16",
-    "F17"                   : "F17",
-    "F18"                   : "F18",
-    "F19"                   : "F19",
-    "F2"                    : "F2",
-    "F20"                   : "F20",
-    "F21"                   : "F21",
-    "F22"                   : "F22",
-    "F23"                   : "F23",
-    "F24"                   : "F24",
-    "F3"                    : "F3",
-    "F4"                    : "F4",
-    "F5"                    : "F5",
-    "F6"                    : "F6",
-    "F7"                    : "F7",
-    "F8"                    : "F8",
-    "F9"                    : "F9",
-    "FINAL"                 : "Final",
-    "HELP"                  : "Help",
-    "HOME"                  : "Home",
-    "ICO_00"                : "Ico00",
-    "INSERT"                : "Insert",
-    "JUNJA"                 : "Junja",
-    "KANA"                  : "Kana",
-    "KANJI"                 : "Kanji",
-    "LAUNCH_APP1"           : "XF86LaunchA",
-    "LAUNCH_APP2"           : "XF86LaunchB",
-    "LAUNCH_MAIL"           : "XF86Mail",
-    "LAUNCH_MEDIA_SELECT"   : "XF86AudioMedia",
-    "LCONTROL"              : "Control_L",
-    "LEFT"                  : "Left",
-    "LMENU"                 : "Alt_L",
-    "LSHIFT"                : "Shift_L",
-    "LWIN"                  : "Super_L",
-    "MBUTTON"               : "Middle",
-    "MEDIA_NEXT_TRACK"      : "Next",
-    "MEDIA_PLAY_PAUSE"      : "Play",
-    "MEDIA_PREV_TRACK"      : "Previous",
-    "MEDIA_STOP"            : "Stop",
-    "MODECHANGE"            : "Mode",
-    "NEXT"                  : "Page",
-    "NONCONVERT"            : "Non",
-    "NUMLOCK"               : "Num",
-    "OEM_FJ_JISHO"          : "Jisho",
-    "PAUSE"                 : "Pause",
-    "PRINT"                 : "Print",
-    "PRIOR"                 : "Page",
-    "RCONTROL"              : "Control_R",
-    "RIGHT"                 : "Right",
-    "RMENU"                 : "Alt_R",
-    "RSHIFT"                : "Shift_R",
-    "RWIN"                  : "Super_R",
-    "SCROLL"                : "Scroll_Lock",
-    "SLEEP"                 : "XF86Sleep",
-    "SNAPSHOT"              : "Print",
-    "UP"                    : "Up",
-    "VOLUME_DOWN"           : "XF86AudioLowerVolume",
-    "VOLUME_MUTE"           : "XF86AudioMute",
-    "VOLUME_UP"             : "XF86AudioRaiseVolume",
-    "XBUTTON1"              : "X1",
-    "XBUTTON2"              : "X2",
-}
+    ("BROWSER_FAVORITES",       "XF86Favorites"),
+    ("BROWSER_FORWARD",         "XF86Forward"),
+    ("BROWSER_HOME",            "XF86HomePage"),
+    ("BROWSER_REFRESH",         "XF86Reload"),
+    ("BROWSER_SEARCH",          "XF86Search"),
+    ("BROWSER_STOP",            "XF86Suspend"),
+    ("CAPITAL",                 "Caps_Lock"),
+    ("CONVERT",                 "Convert"),
+    ("DELETE",                  "Delete"),
+    ("DOWN",                    "Down"),
+    ("END",                     "End"),
+    ("F1",                      "F1"),
+    ("F10",                     "F10"),
+    ("F11",                     "F11"),
+    ("F12",                     "F12"),
+    ("F13",                     "F13"),
+    ("F14",                     "F14"),
+    ("F15",                     "F15"),
+    ("F16",                     "F16"),
+    ("F17",                     "F17"),
+    ("F18",                     "F18"),
+    ("F19",                     "F19"),
+    ("F2",                      "F2"),
+    ("F20",                     "F20"),
+    ("F21",                     "F21"),
+    ("F22",                     "F22"),
+    ("F23",                     "F23"),
+    ("F24",                     "F24"),
+    ("F3",                      "F3"),
+    ("F4",                      "F4"),
+    ("F5",                      "F5"),
+    ("F6",                      "F6"),
+    ("F7",                      "F7"),
+    ("F8",                      "F8"),
+    ("F9",                      "F9"),
+    ("FINAL",                   "Final"),
+    ("HELP",                    "Help"),
+    ("HOME",                    "Home"),
+    ("ICO_00",                  "Ico00"),
+    ("INSERT",                  "Insert"),
+    ("JUNJA",                   "Junja"),
+    ("KANA",                    "Kana"),
+    ("KANJI",                   "Kanji"),
+    ("LAUNCH_APP1",             "XF86LaunchA"),
+    ("LAUNCH_APP2",             "XF86LaunchB"),
+    ("LAUNCH_MAIL",             "XF86Mail"),
+    ("LAUNCH_MEDIA_SELECT",     "XF86AudioMedia"),
+    ("LCONTROL",                "Control_L"),
+    ("LEFT",                    "Left"),
+    ("LMENU",                   "Alt_L"),
+    ("LSHIFT",                  "Shift_L"),
+    ("LWIN",                    "Super_L"),
+    ("MBUTTON",                 "Middle"),
+    ("MEDIA_NEXT_TRACK",        "Next"),
+    ("MEDIA_PLAY_PAUSE",        "Play"),
+    ("MEDIA_PREV_TRACK",        "Previous"),
+    ("MEDIA_STOP",              "Stop"),
+    ("MODECHANGE",              "Mode"),
+    ("NEXT",                    "Page"),
+    ("NONCONVERT",              "Non"),
+    ("NUMLOCK",                 "Num"),
+    ("OEM_FJ_JISHO",            "Jisho"),
+    ("PAUSE",                   "Pause"),
+    ("PRINT",                   "Print"),
+    ("PRIOR",                   "Page"),
+    ("RCONTROL",                "Control_R"),
+    ("RIGHT",                   "Right"),
+    ("RMENU",                   "Alt_R"),
+    ("RSHIFT",                  "Shift_R"),
+    ("RWIN",                    "Super_R"),
+    ("SCROLL",                  "Scroll_Lock"),
+    ("SLEEP",                   "XF86Sleep"),
+    ("SNAPSHOT",                "Print"),
+    ("UP",                      "Up"),
+    ("VOLUME_DOWN",             "XF86AudioLowerVolume"),
+    ("VOLUME_MUTE",             "XF86AudioMute"),
+    ("VOLUME_UP",               "XF86AudioRaiseVolume"),
+    ("XBUTTON1",                "X1"),
+    ("XBUTTON2",                "X2"),
+    ]
 
 KEYSYM_DEFS = {
                "bracketleft"        : u"[",
@@ -385,7 +386,7 @@ log("VK_NAMES=%s", VK_NAMES)
 
 #lookup the constants:
 KEYCODES = {}
-for vk, name in VIRTUAL_KEYS.items():
+for vk, name in VIRTUAL_KEYS:
     vk_name = "VK_%s" % vk
     if hasattr(win32con, vk_name):
         val = getattr(win32con, vk_name)
