@@ -204,7 +204,7 @@ class Win32RootWindowModel(RootWindowModel):
         finally:
             pass
         assert pixels, "no pixels returned from GetBitmapBits"
-        v = ImageWrapper(0, 0, width, height, pixels, "BGRX", 24, width*4, planes=ImageWrapper.PACKED, thread_safe=True)
+        v = ImageWrapper(x, y, width, height, pixels, "BGRX", 24, width*4, planes=ImageWrapper.PACKED, thread_safe=True)
         if logger==None:
             logger = log
         log("get_image%s=%s took %ims", (x, y, width, height), v, (time.time()-start)*1000)
@@ -229,6 +229,10 @@ class ShadowServer(GTKShadowServerBase):
     def __init__(self):
         GTKShadowServerBase.__init__(self)
         self.keycodes = {}
+        from xpra.net.bytestreams import set_continue_wait
+        #on win32, we want to wait just a little while,
+        #to prevent servers spinning wildly on non-blocking sockets:
+        set_continue_wait(5)
 
 
     def add_listen_socket(self, socktype, sock):
