@@ -31,7 +31,7 @@ def get_native_system_tray_classes():
         traylog("cannot load appindicator tray: %s", e)
     return []
 
-def get_native_tray_classes():
+def get_wm_name():
     wm_name = os.environ.get("XDG_CURRENT_DESKTOP", "")
     try:
         wm_check = _get_X11_root_property("_NET_SUPPORTING_WM_CHECK", "WINDOW")
@@ -43,11 +43,14 @@ def get_native_tray_classes():
     except Exception as e:
         traylog.error("Error accessing window manager information:")
         traylog.error(" %s", e)
+    return wm_name
+
+def get_native_tray_classes():
     #could restrict to only DEs that have a broken system tray like "GNOME Shell"?
     if has_gtk_menu_support():  #and wm_name=="GNOME Shell":
         try:
             from xpra.platform.xposix.gtkmenu_tray import GTKMenuTray
-            traylog("using GTKMenuTray for %s", wm_name)
+            traylog("using GTKMenuTray for '%s' window manager", get_wm_name() or "unknown")
             return [GTKMenuTray]
         except Exception as e:
             traylog("cannot load gtk menu tray: %s", e)
@@ -93,7 +96,8 @@ def _get_X11_root_property(name, req_type):
         root_xid = window_bindings.getDefaultRootWindow()
         return _get_X11_window_property(root_xid, name, req_type)
     except Exception as e:
-        log.warn("failed to get X11 root property %s: %s", name, e)
+        log.warn("Warning: failed to get X11 root property '%s'", name)
+        log.warn(" %s", e)
     return None
 
 
@@ -456,11 +460,14 @@ def get_info():
     s = _get_xsettings()
     if s:
         serial, values = s
-        i["xsettings.serial"] = serial
+        xi = {"serial"  : serial}
         for _,name,value,_ in values:
-            i["xsettings.%s" % name] = value
-    i["dpi.xsettings"] = _get_xsettings_dpi()
-    i["dpi.randr"] = _get_randr_dpi()
+            xi[name] = value
+        i["xsettings"] = xi
+    i.setdefault("dpi", {
+                         "xsettings"    : _get_xsettings_dpi(),
+                         "randr"        : _get_randr_dpi()
+                         })
     return i
 
 
@@ -477,6 +484,9 @@ class ClientExtras(object):
             self.setup_xprops()
         self.setup_dbus_signals()
 
+    def ready(self):
+        pass
+
     def cleanup(self):
         log("cleanup() xsettings_watcher=%s, root_props_watcher=%s", self._xsettings_watcher, self._root_props_watcher)
         if self._xsettings_watcher:
