@@ -168,6 +168,11 @@ def xpra_runner_shell_script(xpra_file, starting_dir, socket_dir):
         # :-separated envvars that people might change while their server is
         # going:
         if var in ("PATH", "LD_LIBRARY_PATH", "PYTHONPATH"):
+            #prevent those paths from accumulating the same values multiple times,
+            #only keep the first one:
+            pval = value.split(os.pathsep)      #ie: ["/usr/bin", "/usr/local/bin", "/usr/bin"]
+            seen = set()
+            value = os.pathsep.join(x for x in pval if not (x in seen or seen.add(x)))
             script.append("%s=%s:\"$%s\"; export %s\n"
                           % (var, sh_quotemeta(value), var, var))
         else:
@@ -426,6 +431,10 @@ def setup_server_socket_path(dotxpra, sockpath, local_display_name, clobber, wai
 
 
 def setup_local_sockets(bind, socket_dir, socket_dirs, display_name, clobber, mmap_group, socket_permissions):
+    if not bind:
+        return []
+    if not socket_dir and (not socket_dirs or (len(socket_dirs)==1 and not socket_dirs[0])):
+        raise InitException("at least one socket directory must be set to use unix domain sockets")
     dotxpra = DotXpra(socket_dir or socket_dirs[0])
     try:
         dotxpra.mksockdir()
