@@ -8,8 +8,8 @@ def init(opts):
     pass
 
 class Authenticator(object):
-    def __init__(self, username):
+    def __init__(self, username="", **kwargs):
         raise Exception("failing")
 
     def __repr__(self):
-        return "Fail Authenticator"
+        return "fail"
