@@ -10,7 +10,8 @@ assert init
 
 class Authenticator(SysAuthenticator):
 
-    def __init__(self, username):
+    def __init__(self, username, **kwargs):
+        SysAuthenticator.__init__(self, username, **kwargs)
         self.salt = None
         self.pw = None
         self.username = get_username()
@@ -28,4 +29,4 @@ class Authenticator(SysAuthenticator):
         return True
 
     def __repr__(self):
-        return "None Authenticator"
+        return "none"
