@@ -11,7 +11,7 @@ def init(opts):
 
 
 class Authenticator(object):
-    def __init__(self, username):
+    def __init__(self, username, **kwargs):
         pass
 
     def requires_challenge(self):
@@ -36,4 +36,4 @@ class Authenticator(object):
         return None
 
     def __repr__(self):
-        return "Reject Authenticator"
+        return "reject"
