@@ -62,7 +62,7 @@ class SysAuthenticator(object):
 
     def authenticate(self, challenge_response, client_salt):
         #this will call check(password)
-        self.authenticate_check(challenge_response, client_salt)
+        return self.authenticate_check(challenge_response, client_salt)
 
     def authenticate_check(self, challenge_response, client_salt):
         if self.salt is None:
@@ -119,7 +119,10 @@ class SysAuthenticator(object):
         try:
             sockdir = DotXpra(socket_dir, socket_dirs, actual_username=self.username)
             results = sockdir.sockets(check_uid=uid)
-            displays = [display for state, display in results if state==DotXpra.LIVE]
+            displays = []
+            for state, display in results:
+                if state==DotXpra.LIVE and display not in displays:
+                    displays.append(display)
         except Exception as e:
             log.error("Error: cannot get socket directory for '%s':", self.username)
             log.error(" %s", e)
