@@ -28,18 +28,19 @@ import xpra
 from xpra.server import ClientException
 from xpra.scripts.main import SOCKET_TIMEOUT, _socket_connect
 from xpra.scripts.server import deadly_signal
+from xpra.scripts.config import InitException
 from xpra.net.bytestreams import SocketConnection, pretty_socket, set_socket_timeout
 from xpra.platform import set_name
 from xpra.os_util import load_binary_file, get_machine_id, get_user_uuid, platform_name, SIGNAMES, Queue
 from xpra.version_util import version_compat_check, get_version_info_full, get_platform_info, get_host_info, local_version
 from xpra.net.protocol import Protocol, get_network_caps, sanity_checks
-from xpra.net.crypto import new_cipher_caps, ENCRYPTION_CIPHERS, ENCRYPT_FIRST_PACKET, DEFAULT_IV, DEFAULT_SALT, DEFAULT_ITERATIONS, INITIAL_PADDING, DEFAULT_PADDING,\
-    ALL_PADDING_OPTIONS
+from xpra.net.crypto import crypto_backend_init, new_cipher_caps, get_salt, \
+        ENCRYPTION_CIPHERS, ENCRYPT_FIRST_PACKET, DEFAULT_IV, DEFAULT_SALT, DEFAULT_ITERATIONS, INITIAL_PADDING, DEFAULT_PADDING, ALL_PADDING_OPTIONS
 from xpra.server.background_worker import stop_worker, get_worker
 from xpra.make_thread import make_thread
 from xpra.scripts.fdproxy import XpraProxy
 from xpra.server.control_command import ControlError, HelloCommand, HelpCommand, DebugControl
-from xpra.util import typedict, updict, repr_ellipsized, \
+from xpra.util import csv, merge_dicts, typedict, notypedict, flatten_dict, parse_simple_dict, repr_ellipsized, dump_all_frames, \
         SERVER_SHUTDOWN, SERVER_UPGRADE, LOGIN_TIMEOUT, DONE, PROTOCOL_ERROR, SERVER_ERROR, VERSION_ERROR, CLIENT_REQUEST
 
 main_thread = threading.current_thread()
@@ -50,17 +51,19 @@ SIMULATE_SERVER_HELLO_ERROR = os.environ.get("XPRA_SIMULATE_SERVER_HELLO_ERROR",
 
 def get_server_info():
     #this function is for non UI thread info
-    info = {}
+    info = {
+            "platform"  : get_platform_info(),
+            "build"     : get_version_info_full(),
+            }
     info.update(get_host_info())
-    def up(prefix, d):
-        updict(info, prefix, d)
-    up("platform",  get_platform_info())
-    up("build",     get_version_info_full())
     return info
 
 def get_thread_info(proto=None, protocols=[]):
     #threads:
-    info_threads = proto.get_threads()
+    if proto:
+        info_threads = proto.get_threads()
+    else:
+        info_threads = []
     info = {
             "count"        : threading.active_count() - len(info_threads),
             "info.count"   : len(info_threads)
@@ -73,9 +76,10 @@ def get_thread_info(proto=None, protocols=[]):
     if w:
         thread_ident[w.ident] = "worker"
 
+    it = info.setdefault("info", {})
     #threads used by the "info" client:
     for i, t in enumerate(info_threads):
-        info["info[%s]" % i] = t.getName()
+        it[i] = t.getName()
         thread_ident[t.ident] = t.getName()
     for p in protocols:
         try:
@@ -85,8 +89,9 @@ def get_thread_info(proto=None, protocols=[]):
         except:
             pass
     #all non-info threads:
+    anit = info.setdefault("thread", {})
     for i, t in enumerate((x for x in threading.enumerate() if x not in info_threads)):
-        info[str(i)] = t.getName()
+        anit[i] = t.getName()
     #platform specific bits:
     try:
         from xpra.platform.info import get_sys_info
@@ -100,14 +105,15 @@ def get_thread_info(proto=None, protocols=[]):
                 return ""
             return x
         frames = sys._current_frames()
+        fi = info.setdefault("frame", {})
         for i,frame_pair in enumerate(frames.items()):
             stack = traceback.extract_stack(frame_pair[1])
-            info["frame[%s].thread" % i] = thread_ident.get(frame_pair[0], "unknown")
             #sanitize stack to prevent None values (which cause encoding errors with the bencoder)
             sanestack = []
             for e in stack:
                 sanestack.append(tuple([nn(x) for x in e]))
-            info["frame[%s].stack" % i] = sanestack
+            fi[i] = {""         : thread_ident.get(frame_pair[0], "unknown"),
+                     "stack"    : sanestack}
     except Exception as e:
         log.error("failed to get frame info: %s", e)
     return info
@@ -124,16 +130,20 @@ class ServerCore(object):
     EXITING_CODE = 2
 
     def __init__(self):
+        crypto_backend_init()
         log("ServerCore.__init__()")
         self.start_time = time.time()
         self.auth_class = None
         self.tcp_auth_class = None
+        self.vsock_auth_class = None
         self._when_ready = []
         self.child_reaper = None
+        self.original_desktop_display = None
 
         self._closing = False
         self._upgrading = False
         #networking bits:
+        self._socket_info = []
         self._potential_protocols = []
         self._tcp_proxy_clients = []
         self._tcp_proxy = ""
@@ -143,12 +153,11 @@ class ServerCore(object):
         self._max_connections = MAX_CONCURRENT_CONNECTIONS
         self._socket_timeout = 0.1
         self._socket_dir = None
-        self.main_socket_path = ""
+        self.unix_socket_paths = []
 
         self.session_name = ""
 
         #Features:
-        self.digest_modes = ("hmac", )
         self.encryption = None
         self.encryption_keyfile = None
         self.tcp_encryption = None
@@ -178,7 +187,7 @@ class ServerCore(object):
         self.session_name = opts.session_name
         set_name("Xpra", self.session_name or "Xpra")
 
-        self.main_socket_path = ""
+        self.unix_socket_paths = []
         self._socket_dir = opts.socket_dir or opts.socket_dirs[0]
         self._tcp_proxy = opts.tcp_proxy
         self.encryption = opts.encryption
@@ -193,43 +202,44 @@ class ServerCore(object):
         self.init_auth(opts)
 
     def init_auth(self, opts):
-        self.auth_class = self.get_auth_module("unix-domain-socket", opts.auth, opts)
-        self.tcp_auth_class = self.get_auth_module("tcp-socket", opts.tcp_auth or opts.auth, opts)
-        authlog("init_auth(%s) auth class=%s, tcp auth class=%s", opts, self.auth_class, self.tcp_auth_class)
+        self.auth_class = self.get_auth_module("unix-domain", opts.auth, opts)
+        self.tcp_auth_class = self.get_auth_module("tcp", opts.tcp_auth or opts.auth, opts)
+        self.vsock_auth_class = self.get_auth_module("vsock", opts.vsock_auth, opts)
+        authlog("init_auth(..) auth class=%s, tcp auth class=%s, vsock auth class=%s", self.auth_class, self.tcp_auth_class, self.vsock_auth_class)
 
-    def get_auth_module(self, socket_type, auth, opts):
-        authlog("get_auth_module(%s, %s, %s)", socket_type, auth, opts)
-        if not auth and opts.password_file:
-            authlog.warn("Warning: 'password-file' used without an authentication module for %ss", socket_type)
-            authlog.warn(" using 'file' based authentication")
-            auth = "file"
-        if not auth and os.environ.get('XPRA_PASSWORD'):
-            authlog.warn("no authentication module specified for %ss with 'XPRA_PASSWORD'", socket_type)
-            authlog.warn(" using 'file' based authentication")
-            auth = "file"
-        if auth=="":
+    def get_auth_module(self, socket_type, auth_str, opts):
+        authlog("get_auth_module(%s, %s, {..})", socket_type, auth_str)
+        if not auth_str:
             return None
-        elif auth=="sys":
+        #separate options from the auth module name
+        parts = auth_str.split(":", 1)
+        auth = parts[0]
+        auth_options = {}
+        if len(parts)>1:
+            auth_options = parse_simple_dict(parts[1])
+        if auth=="sys":
             #resolve virtual "sys" auth:
             if sys.platform.startswith("win"):
                 auth = "win32"
             else:
                 auth = "pam"
             authlog("will try to use sys auth module '%s' for %s", auth, sys.platform)
-        from xpra.server.auth import fail_auth, reject_auth, allow_auth, none_auth, file_auth
+        from xpra.server.auth import fail_auth, reject_auth, allow_auth, none_auth, file_auth, multifile_auth, password_auth, env_auth
         AUTH_MODULES = {
                         "fail"      : fail_auth,
                         "reject"    : reject_auth,
                         "allow"     : allow_auth,
                         "none"      : none_auth,
-                        "file"      : file_auth
+                        "env"       : env_auth,
+                        "password"  : password_auth,
+                        "multifile" : multifile_auth,
+                        "file"      : file_auth,
                         }
         try:
             from xpra.server.auth import pam_auth
             AUTH_MODULES["pam"] = pam_auth
         except Exception as e:
             authlog("cannot load pam auth: %s", e)
-        from xpra.scripts.config import InitException
         if sys.platform.startswith("win"):
             try:
                 from xpra.server.auth import win32_auth
@@ -237,26 +247,29 @@ class ServerCore(object):
             except Exception as e:
                 authlog.error("Error: cannot load the MS Windows authentication module:")
                 authlog.error(" %s", e)
-        auth_module = AUTH_MODULES.get(auth.lower())
+        auth_module = AUTH_MODULES.get(auth)
         if not auth_module:
-            raise InitException("cannot find authentication module '%s' (supported: %s)" % (auth, AUTH_MODULES.keys()))
+            raise InitException("cannot find authentication module '%s' (supported: %s)" % (auth, csv(AUTH_MODULES.keys())))
         try:
             auth_module.init(opts)
             auth_class = getattr(auth_module, "Authenticator")
-            return auth_class
+            auth_class.auth_name = auth.lower()
+            return auth, auth_class, auth_options
         except Exception as e:
             raise InitException("Authenticator class not found in %s" % auth_module)
 
 
     def init_sockets(self, sockets):
+        self._socket_info = sockets
         ### All right, we're ready to accept customers:
         for socktype, sock, info in sockets:
             netlog("init_sockets(%s) will add %s socket %s (%s)", sockets, socktype, sock, info)
             self.idle_add(self.add_listen_socket, socktype, sock)
             if socktype=="unix-domain" and info:
                 try:
-                    self.main_socket_path = os.path.abspath(info)
-                    netlog("main socket path: %s", self.main_socket_path)
+                    p = os.path.abspath(info)
+                    self.unix_socket_paths.append(p)
+                    netlog("added unix socket path: %s", p)
                 except Exception as e:
                     log.error("failed to set socket path to %s: %s", info, e)
 
@@ -330,24 +343,16 @@ class ServerCore(object):
             os_util.force_quit()
         self.timeout_add(5000, force_quit)
         log("clean_quit(..) quit timers scheduled")
-        self.log_exit_state()
+        dump_all_frames()
 
-    def log_exit_state(self):
-        frames = sys._current_frames()
-        log("clean_quit() after cleanup, found %s frames:", len(frames))
-        for i,(fid,frame) in enumerate(frames.items()):
-            log("%i: %s - %s:", i, fid, frame)
-            for x in traceback.format_stack(frame):
-                for l in x.splitlines():
-                    log("%s", l)
-
-    def quit(self, upgrading):
+    def quit(self, upgrading=False):
         log("quit(%s)", upgrading)
         self._upgrading = upgrading
         log.info("xpra is terminating.")
         sys.stdout.flush()
         self.do_quit()
         log("quit(%s) do_quit done!", upgrading)
+        dump_all_frames()
 
     def do_quit(self):
         raise NotImplementedError()
@@ -358,8 +363,9 @@ class ServerCore(object):
     def run(self):
         self.print_run_info()
         self.print_screen_info()
-        signal.signal(signal.SIGTERM, self.signal_quit)
+        #SIGINT breaks GTK3.. (but there are no py3k servers yet!)
         signal.signal(signal.SIGINT, self.signal_quit)
+        signal.signal(signal.SIGTERM, self.signal_quit)
         def start_ready_callbacks():
             for x in self._when_ready:
                 try:
@@ -445,7 +451,12 @@ class ServerCore(object):
             return False
         socktype = self.socket_types.get(listener)
         assert socktype, "cannot find socket type for %s" % listener
-        sock, address = listener.accept()
+        try:
+            sock, address = listener.accept()
+        except socket.error as e:
+            netlog.error("Error: cannot accept new connection:")
+            netlog.error(" %s", e)
+            return True
         if len(self._potential_protocols)>=self._max_connections:
             netlog.error("too many connections (%s), ignoring new one", len(self._potential_protocols))
             sock.close()
@@ -458,15 +469,18 @@ class ServerCore(object):
         target = peername or sockname
         sock.settimeout(self._socket_timeout)
         netlog("new_connection(%s) sock=%s, timeout=%s, sockname=%s, address=%s, peername=%s", args, sock, self._socket_timeout, sockname, address, peername)
-        sc = SocketConnection(sock, sockname, address, target, socktype)
-        netlog("socket connection: %s", sc)
+        conn = SocketConnection(sock, sockname, address, target, socktype)
+        netlog("socket connection: %s", conn)
         frominfo = ""
         if peername:
             frominfo = " from %s" % pretty_socket(peername)
         elif socktype=="unix-domain":
             frominfo = " on %s" % sockname
+        return self.make_protocol(socktype, conn, frominfo)
+
+    def make_protocol(self, socktype, conn, frominfo=""):
         netlog.info("New %s connection received%s", socktype, frominfo)
-        protocol = Protocol(self, sc, self.process_packet)
+        protocol = Protocol(self, conn, self.process_packet)
         self._potential_protocols.append(protocol)
         protocol.large_packets.append("info-response")
         protocol.challenge_sent = False
@@ -475,6 +489,10 @@ class ServerCore(object):
             protocol.auth_class = self.tcp_auth_class
             protocol.encryption = self.tcp_encryption
             protocol.keyfile = self.tcp_encryption_keyfile
+        elif socktype=="vsock":
+            protocol.auth_class = self.vsock_auth_class
+            protocol.encryption = None
+            protocol.keyfile = None
         else:
             protocol.auth_class = self.auth_class
             protocol.encryption = self.encryption
@@ -497,7 +515,7 @@ class ServerCore(object):
             #start a new proxy in a thread
             def run_proxy():
                 self.start_tcp_proxy(proto, data)
-            make_thread(run_proxy, "web-proxy-for-%s" % proto).start()
+            make_thread(run_proxy, "web-proxy-for-%s" % proto, daemon=True).start()
             return
         err = "invalid packet format, not an xpra client?"
         proto.gibberish(err, data)
@@ -544,13 +562,15 @@ class ServerCore(object):
             if buf:
                 proxylog("pushing read buffer to its new destination: %s", repr_ellipsized(buf))
                 web_server_connection.write(buf)
-        p = XpraProxy(client_connection.target, client_connection, web_server_connection)
+        p = XpraProxy(client_connection.target, client_connection, web_server_connection, self.tcp_proxy_quit)
         self._tcp_proxy_clients.append(p)
         proxylog.info("client connection from %s forwarded to proxy server on %s:%s", client_connection.target, host, port)
-        p.run()
-        proxylog("run_proxy() %s ended", p)
-        if p in self._tcp_proxy_clients:
-            self._tcp_proxy_clients.remove(p)
+        p.start_threads()
+
+    def tcp_proxy_quit(self, proxy):
+        proxylog("tcp_proxy_quit(%s)", proxy)
+        if proxy in self._tcp_proxy_clients:
+            self._tcp_proxy_clients.remove(proxy)
 
     def is_timedout(self, protocol):
         #subclasses may override this method (ServerBase does)
@@ -586,7 +606,7 @@ class ServerCore(object):
         if extra:
             i += " (%s)" % extra
         try:
-            proto_info = " '%s'" % protocol._conn.get_info().get("endpoint")
+            proto_info = " %s" % protocol._conn.get_info().get("endpoint")
         except:
             proto_info = " %s" % protocol
         netlog.info("Disconnecting client%s:", proto_info)
@@ -652,7 +672,8 @@ class ServerCore(object):
         auth_caps = self.verify_hello(proto, c)
         if auth_caps is not False:
             if c.boolget("info_request", False):
-                self.send_hello_info(proto)
+                flatten = not c.boolget("info-namespace", False)
+                self.send_hello_info(proto, flatten)
                 return
             command_req = c.strlistget("command_request")
             if len(command_req)>0:
@@ -684,7 +705,8 @@ class ServerCore(object):
             return  False
 
         def auth_failed(msg):
-            authlog.warn("Warning: authentication failed: %s", msg)
+            authlog.error("Error: authentication failed")
+            authlog.error(" %s", msg)
             self.timeout_add(1000, self.disconnect_client, proto, msg)
 
         #authenticator:
@@ -692,13 +714,17 @@ class ServerCore(object):
         if proto.authenticator is None and proto.auth_class:
             authlog("creating authenticator %s", proto.auth_class)
             try:
-                proto.authenticator = proto.auth_class(username)
+                auth, aclass, options = proto.auth_class
+                ainstance = aclass(username, **options)
+                proto.authenticator = ainstance
+                authlog("%s=%s", auth, ainstance)
             except Exception as e:
-                authlog.error("Error instantiating %s: %s", proto.auth_class, e)
+                authlog.error("Error instantiating %s:", proto.auth_class)
+                authlog.error(" %s", e)
                 auth_failed("authentication failed")
                 return False
-        self.digest_modes = c.get("digest", ("hmac", ))
 
+        digest_modes = c.get("digest", ("hmac", ))
         #client may have requested encryption:
         cipher = c.strget("cipher")
         cipher_iv = c.strget("cipher.iv")
@@ -719,15 +745,15 @@ class ServerCore(object):
             if padding not in ALL_PADDING_OPTIONS:
                 auth_failed("unsupported padding: %s" % padding)
                 return False
-            authlog("set output cipher using encryption key %s", encryption_key)
+            authlog("set output cipher using encryption key '%s'", repr_ellipsized(encryption_key))
             proto.set_cipher_out(cipher, cipher_iv, encryption_key, key_salt, iterations, padding)
             #use the same cipher as used by the client:
             auth_caps = new_cipher_caps(proto, cipher, encryption_key, padding_options)
             authlog("server cipher=%s", auth_caps)
         else:
             if proto.encryption:
-                authlog.warn("Warning: client does not provide encryption tokens")
-                auth_failed("missing encryption")
+                authlog("client does not provide encryption tokens")
+                auth_failed("missing encryption tokens")
                 return False
             auth_caps = None
 
@@ -740,22 +766,23 @@ class ServerCore(object):
             if not challenge_response:
                 if proto.challenge_sent:
                     auth_failed("invalid state, challenge already sent - no response!")
-                    return False                
+                    return False
                 if proto.authenticator:
                     challenge = proto.authenticator.get_challenge()
                     if challenge is None:
                         auth_failed("invalid state, unexpected challenge response")
                         return False
+                    authlog("challenge: %s", challenge)
                     salt, digest = challenge
-                    authlog.info("Authentication required, %s sending challenge for '%s' using digest %s", proto.authenticator, username, digest)
-                    if digest not in self.digest_modes:
+                    authlog.info("Authentication required by %s authenticator module", proto.authenticator)
+                    authlog.info(" sending challenge for '%s' using %s digest", username, digest)
+                    if digest not in digest_modes:
                         auth_failed("cannot proceed without %s digest support" % digest)
                         return False
                 else:
                     authlog.warn("Warning: client expects a challenge but this connection is unauthenticated")
                     #fake challenge so the client will send the real hello:
-                    from xpra.os_util import get_hex_uuid
-                    salt = get_hex_uuid()+get_hex_uuid()
+                    salt = get_salt()
                     digest = "hmac"
                 proto.challenge_sent = True
                 proto.send_now(("challenge", salt, auth_caps or "", digest))
@@ -781,23 +808,16 @@ class ServerCore(object):
     def get_encryption_key(self, authenticator=None, keyfile=None):
         #if we have a keyfile specified, use that:
         authlog("get_encryption_key(%s, %s)", authenticator, keyfile)
+        v = None
         if keyfile:
             authlog("loading encryption key from keyfile: %s", keyfile)
-            return self.filedata_nocrlf(keyfile)
-        env_key = os.environ.get('XPRA_ENCRYPTION_KEY')
-        if env_key:
-            authlog("using encryption key from %s environment variable", 'XPRA_ENCRYPTION_KEY')
-            return env_key
-        v = None
-        if authenticator:
-            authlog("trying to get encryption key from: %s", authenticator)
+            v = self.filedata_nocrlf(keyfile)
+        if not v:
+            v = os.environ.get('XPRA_ENCRYPTION_KEY')
+            if v:
+                authlog("using encryption key from %s environment variable", 'XPRA_ENCRYPTION_KEY')
+        if not v and authenticator:
             v = authenticator.get_password()
-        if v is None and self.password_file:
-            authlog("trying to load encryption key from password file: %s", self.password_file)
-            v = self.filedata_nocrlf(self.password_file)
-        if v is None and os.environ.get('XPRA_PASSWORD'):
-            authlog("using encryption key from %s environment variable", 'XPRA_PASSWORD')
-            v = os.environ.get('XPRA_PASSWORD')
         return v
 
     def hello_oked(self, proto, packet, c, auth_caps):
@@ -824,7 +844,7 @@ class ServerCore(object):
             v = command.run(*args[1:])
             return 0, v
         except ControlError as e:
-            commandlog.error("error %s processing command %s", e.code, name)
+            commandlog.error("error %s processing control command '%s'", e.code, name)
             msgs = [" %s" % e]
             if e.help:
                 msgs.append(" '%s': %s" % (name, e.help))
@@ -832,8 +852,8 @@ class ServerCore(object):
                 commandlog.error(msg)
             return e.code, "\n".join(msgs)
         except Exception as e:
-            commandlog.error("error processing command %s", name, exc_info=True)
-            return 127, "error processing command: %s" % e
+            commandlog.error("error processing control command '%s'", name, exc_info=True)
+            return 127, "error processing control command: %s" % e
 
 
     def accept_client(self, proto, c):
@@ -864,9 +884,9 @@ class ServerCore(object):
 
     def make_hello(self, source):
         now = time.time()
-        capabilities = get_network_caps()
+        capabilities = flatten_dict(get_network_caps())
         if source.wants_versions:
-            capabilities.update(get_server_info())
+            capabilities.update(flatten_dict(get_server_info()))
         capabilities.update({
                         "version"               : xpra.__version__,
                         "start_time"            : int(self.start_time),
@@ -887,24 +907,36 @@ class ServerCore(object):
         return capabilities
 
 
-    def send_hello_info(self, proto):
+    def send_hello_info(self, proto, flatten=True):
         #Note: this can be overriden in subclasses to pass arguments to get_ui_info()
         #(ie: see server_base)
         log.info("processing info request from %s", proto._conn)
-        self.get_all_info(self.do_send_info, proto)
+        def cb(proto, info):
+            self.do_send_info(proto, info, flatten)
+        self.get_all_info(cb, proto)
 
-    def do_send_info(self, proto, info):
+    def do_send_info(self, proto, info, flatten):
+        if flatten:
+            info = flatten_dict(info)
+        else:
+            info = notypedict(info)
         proto.send_now(("hello", info))
 
-    def get_all_info(self, callback, proto, *args):
+    def get_all_info(self, callback, proto=None, *args):
+        start = time.time()
         ui_info = self.get_ui_info(proto, *args)
+        end = time.time()
+        log("get_all_info: ui info collected in %ims", (end-start)*1000)
         def in_thread(*args):
+            start = time.time()
             #this runs in a non-UI thread
             try:
                 info = self.get_info(proto, *args)
-                ui_info.update(info)
+                merge_dicts(ui_info, info)
             except Exception as e:
                 log.error("error during info collection: %s", e, exc_info=True)
+            end = time.time()
+            log("get_all_info: non ui info collected in %ims", (end-start)*1000)
             callback(proto, ui_info)
         make_thread(in_thread, "Info", daemon=True).start()
 
@@ -916,40 +948,64 @@ class ServerCore(object):
         return get_thread_info(proto)
 
     def get_info(self, proto, *args):
+        start = time.time()
         #this function is for non UI thread info
         info = {}
         def up(prefix, d):
-            updict(info, prefix, d)
+            info[prefix] = d
         filtered_env = os.environ.copy()
         if filtered_env.get('XPRA_PASSWORD'):
             filtered_env['XPRA_PASSWORD'] = "*****"
         if filtered_env.get('XPRA_ENCRYPTION_KEY'):
             filtered_env['XPRA_ENCRYPTION_KEY'] = "*****"
 
-        up("network",   get_network_caps())
-        up("network", {"encryption"     : self.encryption or "",
-                       "tcp-encryption" : self.tcp_encryption or "",
-                       })
-        up("server",    get_server_info())
+        si = get_server_info()
+        si.update({
+                   "mode"              : self.get_server_mode(),
+                   "type"              : "Python",
+                   "start_time"        : int(self.start_time),
+                   "idle-timeout"      : int(self.server_idle_timeout),
+                   "argv"              : sys.argv,
+                   "path"              : sys.path,
+                   "exec_prefix"       : sys.exec_prefix,
+                   "executable"        : sys.executable,
+                })
+        if self.original_desktop_display:
+            si["original-desktop-display"] = self.original_desktop_display
+        up("server", si)
+        from xpra.net.net_util import get_info as get_net_info
+        ni = get_net_info()
+        ni.update({
+                   "sockets"        : self.get_socket_info(),
+                   "encryption"     : self.encryption or "",
+                   "tcp-encryption" : self.tcp_encryption or "",
+                   })
+        up("network", ni)
         up("threads",   self.get_thread_info(proto))
         up("env",       filtered_env)
-        up("server", {
-                "mode"              : self.get_server_mode(),
-                "type"              : "Python",
-                "start_time"        : int(self.start_time),
-                "idle-timeout"      : int(self.server_idle_timeout),
-                "authenticator"     : str((self.auth_class or str)("")),
-                "argv"              : sys.argv,
-                "path"              : sys.path,
-                "exec_prefix"       : sys.exec_prefix,
-                "executable"        : sys.executable,
-                })
         if self.session_name:
-            info["session.name"] = self.session_name
+            info["session"] = {"name" : self.session_name}
         if self.child_reaper:
             info.update(self.child_reaper.get_info())
+        end = time.time()
+        log("ServerCore.get_info took %ims", (end-start)*1000)
         return info
 
+    def get_socket_info(self):
+        si = {}
+        for socktype, _, info in self._socket_info:
+            if info:
+                si.setdefault(socktype, {}).setdefault("listeners", []).append(info)
+        for socktype, auth_class in {
+                                     "tcp"          : self.tcp_auth_class,
+                                     "unix-domain"  : self.auth_class,
+                                     "vsock"        : self.vsock_auth_class,
+                                     }.items():
+            if auth_class:
+                si.setdefault(socktype, {})["authenticator"] = auth_class[0], auth_class[2]
+        return si
+
+
     def process_packet(self, proto, packet):
         try:
             handler = None
