@@ -739,7 +739,7 @@ class ServerSource(object):
             proxy_release = c.strget("proxy.platform.release")
             proxy_version = c.strget("proxy.version")
             proxy_version = c.strget("proxy.build.version", proxy_version)
-            msg = "via %s proxy version %s" % (platform_name(proxy_platform, proxy_release), std(proxy_version))
+            msg = "via %s proxy version %s" % (platform_name(proxy_platform, proxy_release), std(proxy_version or "unknown"))
             if proxy_hostname:
                 msg += " on '%s'" % std(proxy_hostname)
             proxylog.info(msg)
@@ -871,7 +871,7 @@ class ServerSource(object):
             for colorspace, spec_props in colorspace_specs.items():
                 for spec_prop in spec_props:
                     #make a new spec based on spec_props:
-                    spec = video_spec(Encoder)
+                    spec = video_spec(codec_class=Encoder, codec_type="proxy", encoding=encoding)
                     for k,v in spec_prop.items():
                         setattr(spec, k, v)
                     proxylog("parse_proxy_video() adding: %s / %s / %s", encoding, colorspace, spec)
@@ -1129,7 +1129,7 @@ class ServerSource(object):
             cinfo = ""
             if ss:
                 try:
-                    encoder_latency = ss.last_info.get("queue", {}).get("cur", 0)
+                    encoder_latency = ss.info.get("queue", {}).get("cur", 0)
                     avsynclog("server side queue level: %s", encoder_latency)
                     from xpra.sound.gstreamer_util import ENCODER_LATENCY
                     encoder_latency = ENCODER_LATENCY.get(ss.codec, 0)
@@ -1557,8 +1557,9 @@ class ServerSource(object):
             return
         def do_send_cursor():
             self.send_cursor_pending = False
-            cursor_data, cursor_sizes = self.get_cursor_data_cb()
-            if cursor_data:
+            cd = self.get_cursor_data_cb()
+            if cd and cd[0]:
+                cursor_data, cursor_sizes = cd
                 if self.last_cursor_sent and self.last_cursor_sent==cursor_data[:8]:
                     cursorlog("do_send_cursor(..) cursor identical to the last one we sent, nothing to do")
                     return
